<?php
/**
* @package		ConverseKit
* @copyright	Copyright (C) Stack Ideas Sdn Bhd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* ConverseKit is free software. This version may have been modified pursuant
* to the GNU General Public License, and as distributed it includes or
* is derivative of works licensed under the GNU General Public License or
* other free or open source software licenses.
* See COPYRIGHT.php for copyright notices and details.
*/
define('CK_BASE', dirname(dirname(dirname(__DIR__))));

require_once(CK_BASE . '/configuration.php');

class ConversekitIntervals
{
	private $state = null;
	private $host = null;
	private $user = null;
	private $password = null;
	private $database = null;
	private $driver = null;
	private $conn = null;
	private $dbprefix = null;
	private $userId = 0;
	private $timestamp = null;
	private $seconds = null;

	public function __construct()
	{
		$config = new JConfig();

		$this->host = $config->host;
		$this->user = $config->user;
		$this->password = $config->password;
		$this->database = $config->db;
		$this->driver = $config->dbtype;
		$this->dbprefix = $config->dbprefix;

		// Create database connection
		$this->createConnection();

		// When the class is initated, we want to get the values from the request
		$this->state = isset($_POST['state']) ? (string) $_POST['state'] : null;
		$this->userId = isset($_POST['userId']) ? (int) $_POST['userId'] : null;
		$this->timestamp = isset($_POST['timestamp']) ? (string) $_POST['timestamp'] : null;
		$this->seconds = isset($_POST['timer']) ? (int) $_POST['timer'] : null;
		$this->seconds = $this->seconds / 1000;
	}

	/**
	 * Escapes a text
	 *
	 * @since	1.0
	 * @access	public
	 */
	public function escape($value)
	{
		if ($this->driver == 'mysql') {
			return mysql_real_escape_string($value);
		}

		if ($this->driver == 'mysqli') {
			return mysqli_real_escape_string($this->conn, $value);
		}
	}

	/**
	 * Creates a new connection to the database
	 *
	 * @since	1.0
	 * @access	public
	 */
	public function createConnection()
	{
		if ($this->driver == 'mysqli') {
			$this->conn = mysqli_connect($this->host, $this->user, $this->password, $this->database);

			// Check connection
			if (mysqli_connect_errno()) {
				echo "Could not connect: " . mysqli_connect_error();
			}
		}


		if ($this->driver == 'mysql'){
			$this->conn = mysql_connect($this->host, $this->user, $this->password);

			if (!$this->conn) {
				die('Could not connect: ' . mysql_error());
			}

			if (!mysql_select_db($this->database)) {
				die('Could not select database: ' . mysql_error());
			}
		}
	}

	/**
	 * Garbage collection
	 *
	 * @since	1.0
	 * @access	public
	 */
	public function __destruct()
	{
		if ($this->driver == 'mysqli') {
			mysqli_close($this->conn);
		}

		if ($this->driver == 'mysql'){
			mysql_close($this->conn);
		}

		return;
	}

	/**
	 * Point of entry for this script
	 *
	 * @since	1.0
	 * @access	public
	 */
	public function execute()
	{
		$allowed = array('open', 'close');

		if (!in_array($this->state, $allowed)) {
			die('Invalid input provided');
		}


		if ($this->state == 'close') {
			return $this->getCounter();
		}

		if ($this->state == 'open') {
			return $this->checkNewMessage();
		}
	}

	/**
	 * Executes query based on mysql adapter
	 *
	 * @since	1.0
	 * @access	public
	 *
	*/
	public function runQuery($query)
	{
		$result = array();

		if ($this->driver == 'mysqli') {
			$result = mysqli_fetch_assoc($this->conn->query($query));
			return $result;
		}

		// Otherwise we assume that it is mysql
		$result = mysql_query($query);
		$result = mysql_fetch_assoc($result);

		return $result;
	}

	/**
	 * Checks for notification on close state
	 *
	 * @since	1.0
	 * @access	public
	 */
	public function getCounter()
	{
		$query = array();

		$query = "select count(a.`conversation_id`) as `total`";
		$query .= " from `#__social_conversations_message` as a";
		$query .= "		inner join `#__social_conversations_message_maps` as b on a.`id` = b.`message_id`";
		$query .= " 	inner join `#__users` as c on a.`created_by` = c.`id`";
		$query .= " where b.`user_id` = " . $this->escape($this->userId);
		$query .= " and b.`state` = 1";
		$query .= " and b.`isread` = 0";
		$query .= " and a.`created_by` != " . $this->escape($this->userId);

		$query = str_ireplace('#__', $this->dbprefix, $query);
		$result = $this->runQuery($query);

		$total = isset($result['total']) ? $result['total'] : 0;

		return $total;
	}

	/**
	 * Checks for notification on open state
	 *
	 * @since	1.0
	 * @access	public
	 */
	public function checkNewMessage()
	{
		$query = array();
		$query[] = 'SELECT COUNT(1) AS result, min(a.created) as created';
		$query[] = 'FROM `#__social_conversations_message` AS a';
		$query[] = 'INNER JOIN `#__users` AS b ON a.created_by = b.id';
		$query[] = 'INNER JOIN `#__social_conversations_message_maps` AS c ON a.id = c.message_id';
		$query[] = 'WHERE c.`user_id` = ' . $this->escape($this->userId);
		$query[] = 'AND c.isread  = 0';
		$query[] = 'AND a.created_by != ' . $this->escape($this->userId);

		if ($this->timestamp) {
			$query[] = 'AND a.created >= \'' . $this->escape($this->timestamp) . '\'';

		} else {
			$this->timestamp = gmdate('Y-m-d H:i:s');
			$query[] = 'AND a.created >= date_sub(\'' . $this->escape($this->timestamp) . '\', INTERVAL ' . $this->escape($this->seconds) . ' SECOND)';
		}

		$this->timestamp = gmdate('Y-m-d H:i:s');

		$query = implode(' ', $query);
		$query = str_ireplace('#__', $this->dbprefix, $query);

		$result = $this->runQuery($query);

		return $result['result'] . '|' . $this->timestamp . '|' . $result['created'];
	}
}

$lib = new ConversekitIntervals();
echo $lib->execute();
exit;
