EasySocial.module('ck/ck', function($) {

var module = this;

EasySocial.require()
.script('ck/monitoring', 'ck/tabs', 'ck/composer')
.library('moment')
.done(function($) {

EasySocial.Controller('CK', {
	defaultOptions: {
		currentPage: null,
		audio: null,
		currentTimeStamp: null,
		ajaxData: {
			"conversekit": 1,
			"option": "",
			"namespace": "",
			"format": "json",
			"limitstart": 0
		},

		"{button}": "[data-ck-button]",
		"{counter}": "[data-ck-counter]",

		"{window}": "[data-ck-window]",
		"{wrapper}": "[data-ck-wrapper]",

		"{headerActions}": "[data-ck-container-header-actions]",
		"{title}": "[data-ck-container-header-title]",

		"{container}": "[data-ck-container]",
		"{body}": "[data-ck-container-body]",
		"{contents}": "[data-ck-container-body] [data-ck-container-body-contents]",

		// Alerts
		"{alert}": "[data-ck-alert]",
		"{alertMessage}": "[data-ck-alert-message]",
		"{closeAlert}": "[data-ck-alert-close]",

		// APIs
		"{chat}": "[data-ck-chat]",
		"{closeButton}": "[data-ck-mobile-close]"
	}
}, function(self, opts, base, parent, monitoring, tabs, composer) { return {
	init: function() {

		// Initialize items
		if (self.settings('sound')) {
			self.initSound();
		}

		opts.ajaxData.option = opts.component;

		// Implement notification monitoring
		self.monitoring = monitoring = self.element.addController(EasySocial.Controller.CK.Monitoring, {
			"{parent}": self
		});

		// Implement notification monitoring
		self.tabs = tabs = self.element.addController(EasySocial.Controller.CK.Tabs, {
			"{parent}": self
		});

		// Implement message
		self.composer = composer = self.element.addController(EasySocial.Controller.CK.Composer, {
			"{parent}": self
		});

		// Set initial counter
		self.updateCounter(opts.initialCounter);

		if (opts.userId) {
			var windowStatus = self.getWindowStatus();

			if (windowStatus.window == 'open') {
				self.open(windowStatus.page);
			}
		}


		if (opts.isIOS) {
			IOSVersion = self.platform.getIOSVersion(true);
			if (self.platform.isSafari() && IOSVersion == 11) {
				self.window().addClass('view-ios-safari');
			}
		}

		// bind on click event on data-ckit-toggle-on to support ES Mobile integration.
		self.element.on('click.start', '[data-ckit-toggle-on]', function () {
			//simulate button click
			self.button().click();
		});
	},

	history: {
		items: [],

		add: function(page) {
			self.history.items.push(page);
		},

		remove: function(page) {

		}
	},

	alerts: {
		hide: function() {
			self.alert().hide();
		},

		error: function(message) {
			self.alerts.show(message, 'error');
		},

		success: function(message) {
			self.alerts.show(message, 'success');
		},

		show: function(message, className) {
			self.alertMessage().html(message);
			self.alert()
				.switchClass('is-' + className)
				.show();
		}
	},

	initSound: function() {
		var userAgent = navigator.userAgent.toLowerCase();

		// Chromium browsers
		var isChrome = userAgent.indexOf('chrome') >= 0;
		var isEdge = userAgent.indexOf('edg') >= 0;
		var isOpera = userAgent.indexOf('opr') >= 0;

		// Audio context can only be created after there is a user interaction on the page for Chromium browsers
		// Therefore, ck/sound cannot be required at the first place #327
		if (isChrome || isEdge || isOpera) {
			$(document).on('click.ck.audio', function() {
				EasySocial.require().script('ck/sound').done(function($) {
					self.initAudio();

					$(document).off('click.ck.audio');
				});
			});
		} else {
			EasySocial.require().script('ck/sound').done(function($) {
				self.initAudio();
			});
		}
	},

	initAudio: function() {
		// Download audio for sound notification
		if (opts.audio == null) {

			var folder = self.settings('path') + '/plugins/system/conversekit/assets/sounds/';

			ion.sound({
				"sounds": [{
					"name": self.settings('soundFile')
				}],
				"path": folder,
				"preload": true,
				"volume": 1.0
			});
		}

		opts.audio = ion.sound;
	},

	ajax: function(data, ajaxProperties) {
		var url = self.settings('path');

		if (self.settings('useIndexForXhr')) {
			url += '/index.php';
		}

		var token = self.settings('esToken');
		var data = $.extend({}, opts.ajaxData, data);

		// Insert the token
		data[token] = 1;

		var props = {
			"method": "post",
			"url": url,
			"data": data
		};

		if (ajaxProperties !== undefined) {
			props = $.extend({}, props, ajaxProperties);
		}

		var ajax = $.ajax(props);

		return ajax;
	},

	loading: function() {
		self.body().addClass('is-loading');
		self.contents().html('');
	},

	playSound: function() {
		if (!opts.audio) {
			return;
		}

		return opts.audio.play();
	},

	render: function(page, contents, callback) {

		// Reset all known classes on the body
		self.body().removeClass('is-loading');

		if (callback !== undefined) {
			callback.call(this, contents);
			return;
		}

		self.contents().html(contents);
	},

	scrollToTop: function() {
		$('.ckit-container__bd')
			.stop(true, false)
			.scrollTop(0);
	},

	scrollToBottom: function() {
		var wrapper = $('[data-ck-container-body-contents]');

		wrapper
			.stop(true, false)
			.animate({scrollTop: 200000}, 2000);

		// For non mobile, if manual scrolling is detected, immediately stop the process above. #295
		if (!opts.isIOS) {
			wrapper.on('scroll.ck.conversations', function() {
				wrapper.stop(true, true);
				wrapper.off('scroll.ck.conversations');
			});
		}
	},

	setCurrentPage: function(page) {
		opts.currentPage = page;

		self.setWindowStatus('open', page);

		// Set the current layout-xxx class to use
		self.body().removeClass('is-empty');
		self.window().switchClass('layout-' + page);
	},

	setCurrentTimeStamp: function(timestamp) {
		opts.currentTimeStamp = timestamp;
	},

	getCurrentTimeStamp: function() {
		return opts.currentTimeStamp;
	},

	addClass: function(className) {
		self.window().addClass(className);
	},

	removeClass: function(className) {
		self.window().removeClass(className);
	},

	setClass: function(className) {
		self.window().switchClass(className);
	},

	setTitle: function(title) {
		self.title().html(title);
	},

	settings: function(key) {
		return opts.settings[key];
	},

	setWindowStatus: function(state, page) {

		if (!self.settings('remember')) {
			return;
		}

		localStorage.setItem('chatWindow', state);
		localStorage.setItem('chatWindowPage', page);
	},

	getWindowStatus: function() {
		var state = {
			'window': 'close',
			'page': ''
		};

		if (!self.settings('remember')) {
			return state;
		}

		state.window = localStorage.getItem('chatWindow');
		state.page = localStorage.getItem('chatWindowPage');

		if (state.page == 'message' || state.page == 'register') {
			state.page = 'conversations';
		}

		return state;
	},

	updateCounter: function(count) {

		if (count > 0) {
			self.counter().html(count)
				.css('display', 'block');

			// integration with esmobile
			$('[data-conversation-notify-ckit]').html(count)
				.css('display', 'block');

			return;
		}

		self.counter().css('display', 'none');

		// integration with esmobile
		$('[data-conversation-notify-ckit]')
			.css('display', 'none');

	},

	updateMonitorWaitState: function(state) {
		self.monitoring.options.wait = state;
	},

	isOpen: function() {
		return self.button().hasClass('is-open');
	},


	isConversationOpen: function() {
		return self.window().hasClass('is-viewing-message');
	},

	isConversationListing: function() {
		return self.window().hasClass('layout-conversations');
	},

	open: function(start) {

		// clear counter on bubble
		self.updateCounter(0);
		monitoring.stop();

		// Fix legacy tab names by pluralizing it
		if (start == 'conversation' || start == 'contact' || start == 'setting') {
			start += 's';
		}

		// Render a tab in the background
		tabs.open(start);

		self.window().removeClass('is-hidden');
		self.button().addClass('is-open');

		monitoring.start();

		// Apply scroll locks on the browser
		setTimeout(function() {

			if (opts.isIOS) {
				self.scroll.lock();
				return;
			}

			self.element.css('position', 'relative');
		}, 200);
	},

	close: function() {
		self.setWindowStatus('close', '');

		self.window().addClass('is-hidden');
		self.button().removeClass('is-open');

		setTimeout(function(){

			if (opts.isIOS) {
				self.scroll.unlock();
				return;
			}

			self.element.css('position', '');

		}, 200);
	},

	// device platform
	platform: {

		isIOS: function() {
			var agent = navigator.userAgent.toLowerCase();
			var matches = agent.match(/(iphone|ipod|ipad)/);

			if (matches) {
				return true;
			}

			return false;
		},
		getIOSVersion: function(mainVersion) {
			var version = '';

			if (self.platform.isIOS()) {
				var agents = navigator.userAgent.split('OS ');
				if (agents.length > 1) {
					var tmp = agents[1].split(' ')[0];

					var subs = tmp.split('_');

					if (mainVersion) {
						version = subs[0];
					} else {
						version = subs.join('.')
					}
				}
			}

			return version;
		},
		isSafari: function() {
			var agent = navigator.userAgent.toLowerCase();
			return agent.indexOf("safari") > -1;
		},
		isFirefox: function() {
			var agent = navigator.userAgent.toLowerCase();
			return agent.indexOf("firefox") > -1;
		}

	},

	scroll: {
		original: {},

		lock: function() {

			window.bodyScrollTop = (typeof window.pageYOffset !== 'undefined') ? window.pageYOffset : (document.documentElement || document.body.parentNode || document.body).scrollTop;

			self.scroll.original = {
				'position': self.element.css('position'),
				'left': self.element.css('left'),
				'width': self.element.css('width'),
				'height': self.element.css('height'),
				'overflow': self.element.css('overflow'),
				'top': self.element.css('top')
			};

			self.element.css({
				'position': 'fixed',
				'left': 0,
				'width': '100%',
				'height': '100%',
				'overflow': 'hidden',
				'top': '-' + window.bodyScrollTop + 'px'
			});
		},

		unlock: function() {
			self.element.css(self.scroll.original);

			window.scrollTo(0, window.bodyScrollTop);
		}
	},

	"{self} onConverseKitUpdate": function(element, event, data, curtimestamp, timer) {

		// Ensure that we only play a new notification sound when there is new message
		var currentCount = parseInt(self.counter().html());

		if (!self.isOpen() && data > 0) {

			// Update the current counter
			self.updateCounter(data);
			self.updateMonitorWaitState(false);

			if (data > currentCount) {
				self.playSound();
			}
			return;
		}

		if (data > 0) {
			var namespace = "";
			var id = "";

			if (self.isConversationOpen()) {
				// viewing conversation. we need to get the new messages.
				namespace = 'getNotificationMessage';
				id = self.composer.options.conversationId;

			} else if (self.isConversationListing()) {
				// viewing conversations list.
				namespace = 'getNotificationConversations';
			}

			if (namespace) {
				self.ajax({
					"namespace": namespace,
					"timestamp": curtimestamp,
					"timer": timer,
					"id": id
				}).done(function(response) {
					if (response.lastmodif !== null) {
						tabs.trigger('onNewMessage', [response]);
					}

					self.updateMonitorWaitState(false);
				});
			}
		} else {
			self.updateMonitorWaitState(false);
		}
	},

	"{closeAlert} click": function(element, event) {
		self.alerts.hide();
	},

	"{closeButton} click": function(element, event) {
		self.close();
	},

	"{chat} click": function(element, event) {
		var recipientId = element.data('ck-chat');
		var conversationId = element.data('conversation-id');

		var start = self.settings('defaultTab');

		if (recipientId) {
			start = 'message:' + recipientId;
		} else if (conversationId) {
			start = 'conversations:' + conversationId;
		}

		self.open(start);
	},

	"{button} click": function(element, event) {
		var start = element.data('start');

		if (opts.userId == 0) {
			start = 'register';
		}

		if (self.isOpen()) {
			self.close();
			return;
		}

		self.open(start);
		return;
	}
}});

module.resolve();

});

});
