EasySocial.module('ck/composer', function($) {

var module = this;

EasySocial.Controller('CK.Composer', {
	defaultOptions: {
		conversationId: null,
		recipientId: null,
		templates: [],

		"{composer}": "[data-ck-composer]",
		"{composerForm}": "[data-ck-composer-form]",
		"{dropzoneWrapper}": "[data-ck-composer-dropzone]",

		"{textarea}": "[data-ck-composer-textarea]",
		"{submit}": "[data-reply-submit]",

		"{contents}": "[data-ck-container-body-contents]",

		// Attachments
		"{attachment}": "[data-ck-composer-attachments]",
		"{attachmentInput}": "[data-ck-composer-attachments-input]",
		"{attachmentList}": "[data-ck-composer-attachments-list]",
		"{attachmentItem}": "[data-ck-attachment-item]",
		"{attachmentTemplate}": "[data-ck-attachment-template]",
		"{removeAttachment}": "[data-ck-attachment-delete]",

		// Giphy
		"{giphyButton}" : "[data-ck-composer-giphy]",
		"{giphyBrowser}" : "[data-ck-giphy-browser]",
		"{giphyItem}" : "[data-ck-composer-giphy-item]",
		"{giphySearch}" : "[data-ck-giphy-search]",
		"{giphyList}" : "[data-ck-giphy-list]",
		"{giphyTrending}" : "[data-ck-giphy-trending]",
		"{giphyLoading}" : "[data-ck-giphy-loading]",

		// Location
		"{location}": "[data-ck-composer-location]",
		"{locationWrapper}": "[data-ck-location-wrapper]",
		"{locationLatitude}": "[data-ck-composer-location-latitude]",
		"{locationLongitude}": "[data-ck-composer-location-longitude]",
		"{locationAddress}": "[data-ck-composer-location-address]",
		"{locationTitle}": "[data-ck-composer-location-title]",
		"{removeLocation}": "[data-ck-location-remove]",

		// Emoji
		"{emoji}": "[data-ck-composer-emoji]",
		"{emojiItem}": "[data-ck-composer-emoji-item]",

		// warnign message wrapper
		"{warningWrapper}" : "[data-ck-warning-wrapper]"
	}
}, function(self, opts, base, parent, ck) { return {
	init: function() {
		ck = parent = self.parent;

		// Initialize attachment template
		if (self.attachmentList().length > 0) {
			opts.templates['attachment'] = $('<div>').html(self.attachmentTemplate()[0]);

			// Remove the template file
			self.attachmentTemplate().remove();
		}
	},

	getTemplate: function(type) {
		var tpl = opts.templates[type];

		var template = tpl.clone();

		return $(template.html());
	},

	insertText: function(text) {
		var val = self.textarea().val();
		val += ' ' + text;

		self.textarea().val(val);
	},

	reset: function() {
		self.textarea().val('');

		self.attachmentList().empty();

		self.clearLocation();
	},

	"{giphySearch} keyup": $.debounce(function(el) {
		// Get the search query
		var query = el.val();

		var loading = self.giphyLoading();
		var giphyList = self.giphyList();
		var trending = self.giphyTrending();

		giphyList.addClass('t-hidden');
		loading.addClass('is-active');

		trending.removeClass('t-hidden');

		if (query != '') {
			trending.addClass('t-hidden');
		}

		var options = {
			'namespace': 'searchGiphy',
			'query': query
		};

		self.loadGiphy(options);
	}, 300),

	isGiphyBrowserHidden: function() {
		return self.giphyBrowser().hasClass('t-hidden');
	},

	hideGiphyBrowser: function() {
		self.giphyBrowser().addClass('t-hidden');
	},

	showGiphyBrowser: function() {
		self.giphyBrowser().removeClass('t-hidden');
	},

	_isGiphyLoaded: false,

	loadGiphy: function(options) {
		var loading = self.giphyLoading();
		var giphyList = self.giphyList();

		parent.ajax(options)
			.done(function(data) {
				loading.removeClass('is-active');
				giphyList.removeClass('t-hidden');

				giphyList.html(data.contents);
			});
	},

	initializeGiphyBrowser: function() {
		// We only initialize once
		if (self._isGiphyLoaded) {
			return;
		}

		self._isGiphyLoaded = true;

		var loading = self.giphyLoading();
		loading.addClass('is-active');

		var options = {
			'namespace': 'searchGiphy',
			'query': ''
		};

		self.loadGiphy(options);
	},

	"{giphyButton} click": function(el, event) {
		self.initializeGiphyBrowser();

		if (self.isGiphyBrowserHidden()) {
			self.showGiphyBrowser();

			return;
		}

		self.hideGiphyBrowser();
	},

	"{giphyItem} click": function(el, event) {
		imageUrl = el.data('original');

		self.insertGiphy(imageUrl);

		// Close the giphy browser
		self.giphyBrowser().addClass('t-hidden')
	},

	insertGiphy: function(url) {
		self.send(imageUrl, true);
	},

	send: function(message, isGiphy) {
		var type = isGiphy ? 'giphy' : 'message';
		var attachments = self.getAttachments();
		var location = self.getLocation();
		var namespace = opts.conversationId ? 'reply' : 'create';

		var options = {
			'namespace': namespace,
			'uid': opts.conversationId,
			'recipientId': opts.recipientId,
			'message': message,
			'type': type
		};

		// Do not send attachement or location when sending a giphy
		if (!isGiphy) {
			if (attachments) {
				options['upload-id'] = attachments;
			}

			if (location) {
				options = $.extend({}, options, location);
			}
		}

		self.hideWarning();

		parent.ajax(options)
			.done(function(output) {

				if (output.code != undefined && output.message != undefined) {
					// has error.
					self.showWarning(output);
					return;
				}

				// Regardless if this is a new conversation or reply, we always set the conversation id.
				opts.conversationId = output.id;

				// When a message is sent, ensure that is-new is removed
				ck.removeClass('is-new');

				if (isGiphy) {
					var item = $(output.contents);

					// Add loader to the giphy image to tell the users that the image hasn't finish load
					item.find('[data-ck-giphy-item-preview-placeholder]').addClass('is-loading');

					item.find('[data-ck-giphy-item-preview]')
						.off('load')
							.on('load', function() {
								item.find('[data-ck-giphy-item-preview-placeholder]').removeClass('is-loading');

								// Need to scroll to bottom again after the image finished loaded
								parent.scrollToBottom();
							});

					// Append message to the content
					parent.contents().append(item);
				} else {
					// Append message to the content
					parent.contents().append(output.contents);
				}

				parent.scrollToBottom();

				// we need to increment the limitstart from conversations controller
				// so that the same message will not be retrieve again.
				parent.tabs.conversations.options.newMessageCounts += 1;

				// update other controller the options.conversationId
				if (namespace == 'create') {
					parent.tabs.conversations.options.conversationId = output.id;
				}

				// If is sent by giphy, we do not need to reset the form
				if (!isGiphy) {
					// Reset the form before the submission happens
					self.reset();
				}
			});
	},

	showWarning: function(err) {
		self.warningWrapper().html(err.message);
		self.warningWrapper().show();
	},

	hideWarning: function() {
		self.warningWrapper().html('');
		self.warningWrapper().hide();
	},

	getAttachments: function() {
		var items = [];

		self.attachmentItem().each(function(i, item) {
			var id = $(item).data('id');

			items.push(id);
		});

		return items;
	},

	getLocation: function() {
		var location = {
			"latitude": self.locationLatitude().val(),
			"longitude": self.locationLongitude().val(),
			"address": self.locationAddress().val()
		};

		if (!location.latitude && !location.longitude) {
			return false;
		}

		return location;
	},

	//
	// Attachments
	//
	insertAttachment: function(attachments) {
		$.each(attachments, function(i, attachment) {
			var form = new FormData();
			form.append('file', attachment, attachment.name, attachment.size, attachment.type);
			form.append('namespace', 'uploadAttachment');
			form.append(ck.settings('esToken'), 1);
			form.append('conversekit', 1);
			form.append('format', 'json');
			form.append('option', ck.options.component);

			self.composerForm().addClass('has-file');

			var template = self.getTemplate('attachment');
			template.find('[data-name]').html(attachment.name);
			template.addClass('is-uploading');

			self.attachmentList().append(template);

			ck.ajax({}, {
				data: form,
				contentType: false,
				processData: false,
				enctype: 'multipart/form-data'
			}).done(function(data) {
				template
					.addClass('is-success')
					.removeClass('is-uploading');

				template.attr('data-id', data.id);

				// Show the attachment error if there is
				if (data.code != undefined && data.message != undefined) {
					self.showWarning(data);
					return;
				}
			}).error(function(data) {
				template
					.addClass('is-error')
					.removeClass('is-uploading');
			});
		});
	},

	"{attachment} click": function(element, event) {
		// Trigger the click event on the file input
		self.attachmentInput().click();

		// Hide the giphy browser if it is enabled as it covers the whole form
		if (!self.isGiphyBrowserHidden()) {
			self.hideGiphyBrowser();
		}
	},

	"{attachmentInput} change": function(element, event) {
		var files = element[0].files;

		self.insertAttachment(files);
	},

	"{removeAttachment} click": function(element, event) {
		var item = element.parents(self.attachmentItem.selector);

		item.remove();
	},

	dropzone: {
		activated: false,

		show: function() {
			self.dropzone.activated = true;
			self.composerForm().addClass('is-dragover');
		},

		hide: function() {
			self.dropzone.activated = false;
			self.composerForm().removeClass('is-dragover');
		}
	},

	"{composerForm} dragover": function(element, event) {
		event.preventDefault();

		// Due to the way drag event is triggered, we need to only bind the necessary event only when the drag is activated
		if (!self.dropzone.activated) {
			self.dropzone.show();

			// Set a slight delay due to ondrag fire events every few miliseconds.
			setTimeout(function() {
				self.composerForm().on('dragleave', function() {
					self.dropzone.hide();

					self.composerForm().off('dragleave');
				});
			}, 25);
		}
	},

	"{composerForm} drop": function(element, event) {
		event.preventDefault();
		event.stopPropagation();

		self.dropzone.hide();

		return false;
	},

	"{self} dropFiles": function(element, event, files) {
		self.insertAttachment(files);

		self.dropzone.hide();
	},

	//
	// Emoji
	//
	"{emoji} click": function(element, event) {
		self.composer().toggleClass('has-icons');

		// Hide the giphy browser if it is enabled as it covers the whole form
		if (!self.isGiphyBrowserHidden()) {
			self.hideGiphyBrowser();
		}
	},

	"{emojiItem} click": function(element, event) {
		var command = element.data('command');

		self.insertText(command);
		self.emoji().click();
	},

	//
	// Location
	//
	clearLocation: function() {
		self.locationLatitude().val('');
		self.locationLongitude().val('');
		self.locationAddress().val('');

		self.locationWrapper().removeClass('is-success is-error');
		self.composerForm().removeClass('has-location is-loading');
	},

	geolocate: function(position) {
		var latitude = position.coords.latitude;
		var longitude = position.coords.longitude;
		var url = "https://maps.googleapis.com/maps/api/geocode/json?latlng=" + latitude + "," + longitude + "&key=" + ck.settings('googleapi');

		// Query google's api
		ck.ajax({}, {
			"url": url,
			"dataType": "json",
			"type": "get"
		}).done(function(data) {

			self.composerForm().removeClass('is-loading');

			if (data.results[0] === undefined) {
				self.locationWrapper().addClass('is-error');

				return;
			}

			self.locationWrapper().addClass('is-success');

			var address = data.results[0].formatted_address;

			self.locationTitle().html(address);
			self.locationLatitude().val(latitude);
			self.locationLongitude().val(longitude);
			self.locationAddress().val(address);
		}).error(function() {
			console.log('error', arguments);
		});
	},

	locateosm: function() {
		self.getUserCoords()
			.then(function(coords){
				EasySocial.ajax('site/controllers/location/getLocations', {
					latitude: coords.latitude,
					longitude: coords.longitude,
				})
				.done(function(results) {
					self.composerForm().removeClass('is-loading');
					result = results[0];

					if (result === undefined) {
						self.locationWrapper().addClass('is-error');

						return;
					}

					self.locationWrapper().addClass('is-success');

					var address = result.formatted_address;

					self.locationTitle().html(address);
					self.locationLatitude().val(coords.latitude);
					self.locationLongitude().val(coords.longitude);
					self.locationAddress().val(address);
				});
			})
			.fail(function(message){
				console.log('error', message);
			});

	},

	getUserCoordsByIP: function() {
		var request =
			$.getJSON("//www.telize.com/geoip?callback=?")
				.then(function(data) {
					return {latitude: data.latitude, longitude: data.longitude};
				});
	},

	userCoords: null,

	getUserCoords: function() {
		var task = $.Deferred();
		var userCoords = self.userCoords;

		// If we have user lat & lng, automatically resolve this.
		if (userCoords) {
			task.resolve(userCoords);
			return task;
		}

		if (navigator.geolocation) {
			navigator.geolocation.getCurrentPosition(

				// If successful
				function(position) {
					var coords = position.coords;

					task.resolve({
						latitude: coords.latitude,
						longitude: coords.longitude
					});
				},

				// If failed, get user coords by IP.
				function() {
					self.getUserCoordsByIP()
						.done(task.resolve)
						.fail(task.reject);
				}

			);
		} else {
			task = self.getUserCoordsByIP();
		}

		return task.done(function(coords){
			self.userCoords = coords;
		});
	},

	"{location} click": function(element, event) {
		// Hide the giphy browser if it is enabled as it covers the whole form
		if (!self.isGiphyBrowserHidden()) {
			self.hideGiphyBrowser();
		}

		self.composerForm().addClass('has-location is-loading');

		if (ck.settings('mapProvider') == 'osm') {
			self.locateosm();
		} else {
			if (navigator.geolocation) {
				navigator.geolocation.getCurrentPosition(self.geolocate);

				return;
			}

			// Try to render the google maps
			EasySocial.require()
			.script($.getScript('https://maps.google.com/maps/api/js?sensor=true&key=' + ck.settings('googleapi')))
			.done(function() {
				navigator.geolocation.getCurrentPosition(self.geolocate);
			});
		}
	},

	"{removeLocation} click": function(element, event) {
		self.clearLocation();
	},

	"{self} onRenderMessage": function(element, event, conversationId, userId) {
		opts.conversationId = conversationId;
		opts.recipientId = userId;

		// hide any error message if any
		self.warningWrapper().hide();
		self.reset();
	},

	"{textarea} keypress": function(element, event) {
		var keycode = (event.keyCode ? event.keyCode : event.which);
		var message = element.val();

		// Capture enter key
		if ((keycode == 13 && !event.shiftKey && ck.settings('submit') == 1) || (keycode == 10 && !event.shiftKey && event.ctrlKey && ck.settings('submit') == 1)) {
			event.preventDefault();
			event.stopPropagation();

			self.send(message);

			// Form needs to be reset after sending the message or it will mess up attachments and location.
			self.reset();

			return false;
		}
	},

	"{textarea} focus": function() {

		self.composer().addClass('is-focus');
		self.toggleTextareFocus(true);
		parent.scrollToBottom();
	},

	"{contents} click": function(element, event) {
		self.composer().removeClass('is-focus');
	},

	"{textarea} blur": function(element, event) {
		self.toggleTextareFocus(false);
	},

	toggleTextareFocus: function(toggle) {
		if (ck.platform.isIOS()) {
			version = ck.platform.getIOSVersion(true);
			if (version == 11 && ck.platform.isSafari()) {
				if (toggle) {
					ck.window().addClass('is-ios-textarea-focus');
				} else {
					ck.window().removeClass('is-ios-textarea-focus');
				}

			}
		}
	},

	"{submit} click": function(element, event) {
		event.preventDefault();

		var message = self.textarea().val();

		self.send(message);
	}
}});

module.resolve();

});
