EasySocial.module('ck/contacts', function($) {

var module = this;

EasySocial.Controller('CK.Contacts', {
	defaultOptions: {

		filters: null,
		limitstart: 0,

		"{item}": "[data-ck-contact-item]",
		"{filter}": "[data-ck-filter]",
		"{searchInput}": "[data-ck-search-input]",
		"{clear}": "[data-ck-search-clear]",
		"{loadMoreButton}": "[data-ck-loadmore]"
	}
}, function(self, opts, base, parent, ck) { return {
	init: function() {
		parent = self.parent;
		ck = self.ck;
	},

	clearSearch: function() {
		self.searchInput().val('');
		ck.setClass('is-viewing');
	},

	isAddingParticipant: function() {

		// Adding new participant
		if (ck.window().hasClass('layout-add-participants') && opts.conversationId) {
			return true;
		}

		return false;
	},

	isViewingParticipant: function() {
		return ck.window().hasClass('is-viewing-participants');
	},

	search: function(text) {
		ck.setClass('is-searching');

		var options = {
			"search": text,
			"limit": ck.settings('contactLimit')
		};

		if (self.isAddingParticipant()) {
			options['addParticipant'] = true;
			options['conversationId'] = opts.conversationId;
		}

		// keep this for pagination use.
		opts.filters = options;

		parent.render('searchContacts', options);
	},

	renderContacts: function(filter) {
		var filter = filter === undefined ? 'all' : filter;
		var options = {
			"filter": filter,
			"limit": ck.settings('contactLimit')
		};

		// Adding new participant
		if (self.isAddingParticipant()) {
			options['addParticipant'] = true;
			options['conversationId'] = opts.conversationId;
		}

		// console.log(options);

		// keep this for pagination use.
		opts.filters = options;

		parent.render('renderContacts', options);

		ck.setClass('is-viewing');
	},

	"{self} onAddParticipant": function(element, event, conversationId) {
		opts.conversationId = conversationId;

		var options = {
			"addParticipant": true,
			"conversationId": opts.conversationId,
			"limit": ck.settings('contactLimit')
		};

		// keep this for pagination use.
		opts.filters = options;

		// reset limitstart
		opts.limitstart = 0;

	},

	"{self} onViewParticipant": function(element, event, conversationId) {
		opts.conversationId = conversationId;

		var options = {
			"filter": "participants",
			"id": opts.conversationId,
			"limit": ck.settings('contactLimit')
		};

		// keep this for pagination use.
		opts.filters = options;

		// reset limitstart
		opts.limitstart = 0;

	},

	"{item} click": function(element, event) {
		var id = element.data('id');
		var namespace = 'message:' + id;

		// When adding participant, clicking on the user shouldn't do anything
		if (self.isAddingParticipant()) {
			return false;
		}

		// In viewing participants mode, clicking on the item should not open new chat
		if (self.isViewingParticipant()) {
			return false;
		}

		parent.open(namespace);
	},

	// loadmore
	loadmore: function(data) {

		if (opts.limitstart == 0) {
			opts.limitstart = data.limit;
		}

		var options = {
			'namespace': 'loadMoreContacts',
			'limitstart': opts.limitstart
		};

		options = $.extend({}, options, data);

		// Apply loading class on the button instead
		self.loadMoreButton().addClass('is-loading');

		ck.ajax(options).done(function(data) {
			self.loadMoreButton().removeClass('is-loading');

			if (data.contents != '') {

				// update contents
				$(data.contents).insertBefore(self.loadMoreButton());

				if (data.nextlimit < 0) {
					self.loadMoreButton().hide();
				} else {
					opts.limitstart = data.nextlimit;
				}

			}

		});

	},

	"{loadMoreButton} click": function() {
		// keep this for pagination use.
		var filters = opts.filters;

		self.loadmore(filters);
	},


	"{searchInput} keyup": $.debounce(function(element, event) {

		var text = element.val();

		// If the user hits the enter key, do not allow browser to propagate the event
		if (event.keyCode == '13') {
			event.stopPropagation();
			event.preventDefault();

			return false;
		}

		if ($.trim(text) == '') {
			self.renderContacts();
			return;
		}

		self.search(text);

	}, 300),

	"{clear} click": function(element, event) {
		var search = $.trim(self.searchInput().val());

		if (search == '') {
			return;
		}

		self.clearSearch();
		self.renderContacts();
	},

	"{self} onRenderTab": function(element, event, tab) {
		if (tab !== 'contacts') {
			return;
		}
		self.clearSearch();
		self.renderContacts();

		opts.limitstart = 0;

		// Set initial filter to all
		self.filter('[data-ck-filter=all]').activateClass('active');
	},

	"{filter} click": function(element, event) {
		var filter = element.data('ck-filter');

		opts.limitstart = 0;

		self.filter('[data-ck-filter=' + filter +']').activateClass('active');
		self.renderContacts(filter);
	}
}});

module.resolve();

});
