EasySocial.module('ck/conversations', function($) {

var module = this;

EasySocial.Controller('CK.Conversations', {
	defaultOptions: {
		selectedItems: [],
		conversationId: null,
		recipientId: null,
		limitstart: 0,
		newMessageCounts: 0,

		"{checkAll}": "[data-ck-checkall]",
		"{massEdit}": "[data-ck-mass-edit]",
		"{massEditDone}": "[data-ck-mass-edit-done]",

		"{conversationItem}": "[data-ck-conversation-item]",
		"{checkboxItems}": "[data-checkbox] input:checkbox",

		"{compose}": "[data-ck-compose]",
		"{item}": "[data-ck-conversation-item]",
		"{toggle}": "[data-inner-toggle]",

		// load more
		"{loadMoreButton}": "[data-ck-loadmore-conversations]",
		"{loadMoreMessageButton}": "[data-ck-loadmore-messages]",

		// Giphy
		"{giphyButton}" : "[data-ck-composer-giphy]",
		"{giphyItem}" : "[data-ck-composer-giphy-item]",

		// Single chat
		"{conversationHeader}": "[data-ck-container-header-message]",
		"{backButton}": "[data-ck-back]",
		"{addParticipant}": "[data-ck-add-participants]",
		"{viewParticipants}": "[data-ck-view-participants]",
		"{insertParticipant}": "[data-ck-insert-participant]",
		"{leave}": "[data-ck-leave-conversations]",
		"{updateTitle}": "[data-ck-change-title]",

		// Title changes
		"{title}": "[data-ck-title]",
		"{titleInput}": "[data-ck-title-input]",
		"{saveTitleButton}": "[data-ck-title-save]",
		"{cancelSaveTitle}": "[data-ck-title-cancel]",

		// Mass actions
		"{actions}": "[data-ck-actions]",
		"{archive}": "[data-ck-archive]",
		"{markRead}": "[data-ck-mark-read]",
		"{delete}": "[data-ck-delete]",
		"{confirmDelete}": "[data-ck-delete-confirm]",
		"{cancelDelete}": "[data-ck-delete-cancel]",

		// Attachments
		"{confirmDeleteAttachment}": "[data-ck-attachment-delete]",
		"{attachmentWrapper}": "[data-ck-attachment-item-wrapper]",
		"{deleteAttachment}": "[data-ck-attachment-item-wrapper] [data-ck-confirm]",
		"{cancelDeleteAttachment}": "[data-ck-attachment-item-wrapper] [data-ck-cancel]"
	}
}, function(self, opts, base, parent, ck) { return {
	init: function() {
		parent = self.parent;
		ck = self.ck;
	},

	hideActions: function() {
		parent.parent.body().removeClass('has-actions');
	},

	showActions: function() {
		// There is no mass actions to add participant in conversations listing
		self.actions().find('[data-ck-add]').addClass('hidden');

		parent.parent.body().addClass('has-actions');
	},

	uncheckAll: function() {
		self.checkboxItems()
			.prop('checked', false)
			.trigger('change');
	},

	renderMessageWindow: function(conversation) {

		if (conversation.canReply) {
			ck.setClass('can-reply');
		} else {
			ck.removeClass('can-reply');
		}

		if (conversation.isNew) {
			ck.setClass('is-new');
		} else {
			ck.removeClass('is-new');
		}

		if (conversation.isGroup) {
			ck.addClass('is-group');
		} else {
			ck.removeClass('is-group');
		}

		// Update the header message
		self.conversationHeader().html(conversation.header);

		// Set the contents
		ck.render('message', conversation.contents);

		// Ensure that the chat always scrolls to the bottom
		ck.scrollToBottom();
	},

	// Renders a conversation with a given user id.
	// ConverseKit will figure out the conversation id.
	renderConversationWithUserId: function(userId) {
		ck.loading();

		ck.setClass('is-viewing-message');
		ck.removeClass('can-reply');

		// Ensure that the contents are always empty so we don't get jumpy effects
		self.conversationHeader().html('');
		ck.setCurrentPage('message');

		ck.ajax({
			"namespace": "renderMessageWithUserId",
			"userId": userId
		}).done(function(conversation) {

			self.renderMessageWindow(conversation);

			// If this is not a new conversation, we should set the conversation id
			opts.conversationId = null;

			if (conversation.id !== undefined) {
				opts.conversationId = conversation.id;
			}

			ck.composer.trigger('onRenderMessage', [opts.conversationId, userId]);
		});
	},

	renderConversation: function(id) {
		ck.loading();

		ck.removeClass('can-reply');
		ck.setClass('is-viewing-message');
		opts.conversationId = id;

		// Ensure that the contents are always empty so we don't get jumpy effects
		self.conversationHeader().html('');
		ck.setCurrentPage('message');

		opts.limitstart = 0;

		ck.ajax({
			"namespace": "renderMessage",
			"id": id,
			"limit": ck.settings('listLimit')
		}).done(function(conversation) {

			self.renderMessageWindow(conversation);

			if (conversation.timestamp != undefined) {
				ck.setCurrentTimeStamp(conversation.timestamp);
			}

			ck.composer.trigger('onRenderMessage', id);
		});
	},

	renderParticipants: function(conversationId) {
		ck.loading();
		ck.setClass('is-viewing-participants');

		parent.contacts.trigger('onViewParticipant', opts.conversationId);

		ck.ajax({
			"namespace": "renderParticipants",
			"id": conversationId,
			"limit": ck.settings('contactLimit')
		}).done(function(contents) {
			ck.render('participants', contents);
		});
	},

	removeSelectedItems: function() {
		$(opts.selectedItems).each(function(i, id) {
			var item = self.conversationItem('[data-id=' + id +']');
			item.remove();
		});

		opts.selectedItems = [];

		// When items are removed, we also need to ensure that everything is unchecked.
		self.uncheckAll();

		// If there is nothing left, we need to add the empty state
		if (self.conversationItem().length <= 0) {
			ck.window().addClass('is-empty');
			ck.window().removeClass('is-editing');
		}
	},

	"{cancelDeleteAttachment} click": function(element, event) {
		var wrapper = element.parents(self.attachmentWrapper.selector);

		wrapper.removeClass('confirm-remove');
	},

	"{deleteAttachment} click": function(element, event) {
		var wrapper = element.parents(self.attachmentWrapper.selector);
		var id = wrapper.data('id');

		ck.ajax({
			'namespace': 'deleteAttachment',
			'fileId': id
		}).done(function() {
			wrapper.remove();
		});
	},

	"{confirmDeleteAttachment} click": function(element, event) {
		var wrapper = element.parents('[data-ck-attachment-item-wrapper]');

		wrapper.addClass('confirm-remove');
	},

	//
	// Custom conversation title
	//
	saveTitle: function(title) {
		ck.removeClass('is-editing');

		ck.ajax({
			"namespace": "updateTitle",
			"conversationId": opts.conversationId,
			"title": title
		}).done(function(response) {
			self.title().html(response.title);

			// Update the original title
			self.titleInput().attr('data-original', response.title);
		});
	},

	"{titleInput} keyup": function(element, event) {
		var title = $.trim(element.val());
		var original = element.data('original');

		if (event.keyCode == '27') {
			element.val(original);
			self.cancelSaveTitle().click();
			return;
		}

		if (event.keyCode == '13' && title != '') {
			self.saveTitle(title);
		}
	},

	"{saveTitleButton} click": function(element, event) {
		var title = self.titleInput().val();

		self.saveTitle(title);
	},

	"{cancelSaveTitle} click": function(element, event) {
		ck.removeClass('is-editing');
	},

	"{updateTitle} click": function(element, event) {
		ck.addClass('is-editing');
	},

	//
	// Participants for group conversations
	//
	"{viewParticipants} click": function(element, event) {
		self.renderParticipants(opts.conversationId);
	},

	"{addParticipant} click": function(element, event) {
		ck.setCurrentPage('add-participants');
		ck.setClass('is-viewing');

		parent.contacts.trigger('onAddParticipant', opts.conversationId);

		// Render the contacts listing for the user
		parent.render('renderContacts', {
			"addParticipant": true,
			"conversationId": opts.conversationId,
			"limit": ck.settings('contactLimit')
		});
	},

	"{insertParticipant} click": function(element, event) {
		event.stopPropagation();

		var contactId = element.data('id');

		ck.ajax({
			"namespace": "addParticipants",
			"conversationId": opts.conversationId,
			"participantId": contactId
		}).done(function(response) {

			// Re-render the conversation
			self.renderConversation(opts.conversationId);
		});
	},

	"{leave} click": function(element, event) {

		ck.ajax({
			"namespace": "leave",
			"conversationId": opts.conversationId
		}).done(function() {

			// Reset the conversation id
			opts.conversationId = null;

			// Render the contacts
			parent.open('conversations');
		});
	},

	"{backButton} click": function(element, event) {

		if (ck.window().hasClass('is-viewing-participants')) {
			self.renderConversation(opts.conversationId);
			return false;
		}

		if (ck.window().hasClass('layout-add-participants')) {
			self.renderConversation(opts.conversationId);

			return false;
		}

		// Always go back to the conversation listing
		ck.open('conversations');
	},

	"{markRead} click": function(element, event) {
		if (opts.selectedItems.length <= 0) {
			return;
		}

		// Fire an ajax call to archive the conversations
		ck.ajax({
			"namespace": "markAsRead",
			"id": opts.selectedItems
		}).done(function(data) {

			self.uncheckAll();
			self.hideActions();

			ck.alerts.success(data.message);
		});
	},

	"{archive} click": function(element, event) {

		if (opts.selectedItems.length <= 0) {
			return;
		}

		// Fire an ajax call to archive the conversations
		ck.ajax({
			"namespace": "archive",
			"id": opts.selectedItems
		}).done(function() {
			self.removeSelectedItems();
		});
	},

	"{delete} click": function(element, event) {
		self.actions().addClass('confirm-remove');
	},

	"{confirmDelete} click": function(element, event) {

		self.actions().removeClass('confirm-remove');

		// Perform an ajax call to delete these conversations
		ck.ajax({
			'namespace': 'deleteConversations',
			'id': opts.selectedItems
		}).done(function(response) {

			self.removeSelectedItems();

			ck.alerts.success(response.message);
		});
	},

	"{cancelDelete} click": function(element, event) {
		self.actions().removeClass('confirm-remove');
	},

	"{checkboxItems} change": function(element, event) {
		var checked = element.is(':checked');
		var id = element.val();

		if (checked) {
			self.showActions();
			opts.selectedItems.push(id);

			return;
		}

		opts.selectedItems = $.grep(opts.selectedItems, function(value) {
								return value != id;
							});

		if (opts.selectedItems.length <= 0) {
			self.hideActions();
		}
	},

	"{checkAll} change": function(element, event) {

		var checked = element.is(':checked');

		if (checked) {
			self.checkboxItems().not(element)
				.prop('checked', checked)
				.trigger('change');

			return;
		}

		self.checkboxItems().not(element)
			.prop('checked', false)
			.trigger('change');
	},

	"{massEditDone} click": function() {
		self.uncheckAll();

		ck.setClass('is-viewing');
	},

	"{massEdit} click": function() {
		ck.setClass('is-editing');
	},

	"{compose} click": function(element, event) {

		// Render the contacts tab so the user can start a new chat
		parent.open('contacts');
	},

	"{item} click": function(element, event) {
		var target = $(event.target);

		// In editing mode, clicking on the item should check the checkbox
		if (ck.window().hasClass('is-editing')) {
			var checked = element.find(self.checkboxItems.selector).is(':checked');

			element.find(self.checkboxItems.selector)
				.prop('checked', !checked)
				.trigger('change');

			return false;
		}

		// Clicking on the dropdown should not fire rendering of chat
		if (target.data('bs-toggle') == 'dropdown' || target.parents('[data-chat-dropdown]').length > 0) {
			return;
		}

		var id = element.data('id');

		self.renderConversation(id);
	},

	"{parent} onNewMessage": function(element, event, response) {

		if (ck.isConversationOpen()) {
			// Append new message to the content
			ck.contents().append(response.contents);
			ck.scrollToBottom();
		} else if (ck.isConversationListing()) {

			if (response.data != undefined && response.total > 0) {

				$.each(response.data, function (i, item) {
					var id = item.conversationId;
					var convItem = $('[data-ck-conversation-item][data-id="' + id + '"]');
					var notify = convItem.find("[data-chat-notify]");
					var currentCount = parseInt(notify.html());

					notify
						.html(item.count)
							.addClass('t-block')
								.removeClass('t-hidden');

					convItem.find("[data-ck-conversation-preview]").html(item.preview);

					convItem.find('[data-ck-conversation-date]').html(item.dateLapse);
				});
			}
		}
	},


	"{self} onRenderTab": function(element, event, tab) {
		if (tab !== 'conversations') {
			return;
		}

		// for pagination
		var options = {
			"limit": ck.settings('listLimit'),
			"sorting": ck.settings('conversationSorting'),
			"ordering": ck.settings('conversationOrdering'),
		};

		parent.render('renderConversations', options, function(output) {
			var ck = this;

			if (output.empty !== undefined) {
				ck.window().addClass('is-empty');
				return;
			}

			ck.contents().html(output.contents);

			ck.setCurrentTimeStamp(output.timestamp);
		});

		opts.limitstart = 0;
		opts.newMessageCounts = 0;

		// Reset the conversation id
		self.conversationId = null;
		ck.setClass('is-viewing');
	},

	// loadmore
	loadmore: function(data, button) {

		if (opts.limitstart == 0) {
			opts.limitstart = data.limit;
		}

		if (opts.newMessageCounts > 0) {
			opts.limitstart += opts.newMessageCounts;
		}

		var options = {
			'limitstart': opts.limitstart
		};

		options = $.extend({}, options, data);

		// Apply loading class on the button instead
		button.addClass('is-loading');

		ck.ajax(options).done(function(data) {
			button.removeClass('is-loading');
			// reset new message count as its no longer needed.
			opts.newMessageCounts = 0;

			if (data.contents != '') {

				if (options.namespace == 'loadMoreConversations') {
					$(data.contents).insertBefore(button);
				} else {
					// update contents
					$(data.contents).insertAfter(button);
				}

				if (data.nextlimit < 0) {
					button.hide();
				} else {
					opts.limitstart = data.nextlimit;
				}
			}

		});

	},

	"{loadMoreButton} click": function (element, event) {

		// this count is not applicable to conversations pagination.
		opts.newMessageCounts = 0;

		// for pagination
		var options = {
			'namespace': 'loadMoreConversations',
			"limit": ck.settings('listLimit')
		};

		self.loadmore(options, element);
	},

	"{loadMoreMessageButton} click": function (element, event) {

		// for pagination
		var options = {
			'namespace': 'loadMoreMessages',
			"limit": ck.settings('listLimit'),
			"id": opts.conversationId
		};

		self.loadmore(options, element);
	}


}});

module.resolve();

});
