EasySocial.module('ck/ck', function($) {

var module = this;

EasySocial.require()
.script('ck/monitoring', 'ck/tabs', 'ck/composer')
.library('moment')
.done(function($) {

EasySocial.Controller('CK', {
	defaultOptions: {
		currentPage: null,
		audio: null,
		currentTimeStamp: null,
		ajaxData: {
			"conversekit": 1,
			"option": "",
			"namespace": "",
			"format": "json",
			"limitstart": 0
		},

		"{button}": "[data-ck-button]",
		"{counter}": "[data-ck-counter]",

		"{window}": "[data-ck-window]",
		"{wrapper}": "[data-ck-wrapper]",

		"{headerActions}": "[data-ck-container-header-actions]",
		"{title}": "[data-ck-container-header-title]",

		"{container}": "[data-ck-container]",
		"{body}": "[data-ck-container-body]",
		"{contents}": "[data-ck-container-body] [data-ck-container-body-contents]",

		// Alerts
		"{alert}": "[data-ck-alert]",
		"{alertMessage}": "[data-ck-alert-message]",
		"{closeAlert}": "[data-ck-alert-close]",

		// APIs
		"{chat}": "[data-ck-chat]",
		"{closeButton}": "[data-ck-mobile-close]"
	}
}, function(self, opts, base, parent, monitoring, tabs, composer) { return {
	init: function() {

		// Initialize items
		if (self.settings('sound')) {
			self.initSound();
		}

		opts.ajaxData.option = opts.component;

		// Implement notification monitoring
		self.monitoring = monitoring = self.element.addController(EasySocial.Controller.CK.Monitoring, {
			"{parent}": self
		});

		// Implement notification monitoring
		self.tabs = tabs = self.element.addController(EasySocial.Controller.CK.Tabs, {
			"{parent}": self
		});

		// Implement message
		self.composer = composer = self.element.addController(EasySocial.Controller.CK.Composer, {
			"{parent}": self
		});

		// Set initial counter
		self.updateCounter(opts.initialCounter);

		if (opts.userId) {
			var windowStatus = self.getWindowStatus();

			if (windowStatus.window == 'open') {
				self.open(windowStatus.page);
			}
		}


		if (opts.isIOS) {
			IOSVersion = self.platform.getIOSVersion(true);
			if (self.platform.isSafari() && IOSVersion == 11) {
				self.window().addClass('view-ios-safari');
			}
		}

		// bind on click event on data-ckit-toggle-on to support ES Mobile integration.
		self.element.on('click.start', '[data-ckit-toggle-on]', function () {
			//simulate button click
			self.button().click();
		});
	},

	history: {
		items: [],

		add: function(page) {
			self.history.items.push(page);
		},

		remove: function(page) {

		}
	},

	alerts: {
		hide: function() {
			self.alert().hide();
		},

		error: function(message) {
			self.alerts.show(message, 'error');
		},

		success: function(message) {
			self.alerts.show(message, 'success');
		},

		show: function(message, className) {
			self.alertMessage().html(message);
			self.alert()
				.switchClass('is-' + className)
				.show();
		}
	},

	initSound: function() {
		var userAgent = navigator.userAgent.toLowerCase();

		// Chromium browsers
		var isChrome = userAgent.indexOf('chrome') >= 0;
		var isEdge = userAgent.indexOf('edg') >= 0;
		var isOpera = userAgent.indexOf('opr') >= 0;

		// Audio context can only be created after there is a user interaction on the page for Chromium browsers
		// Therefore, ck/sound cannot be required at the first place #327
		if (isChrome || isEdge || isOpera) {
			$(document).on('click.ck.audio', function() {
				EasySocial.require().script('ck/sound').done(function($) {
					self.initAudio();

					$(document).off('click.ck.audio');
				});
			});
		} else {
			EasySocial.require().script('ck/sound').done(function($) {
				self.initAudio();
			});
		}
	},

	initAudio: function() {
		// Download audio for sound notification
		if (opts.audio == null) {

			var folder = self.settings('path') + '/plugins/system/conversekit/assets/sounds/';

			ion.sound({
				"sounds": [{
					"name": self.settings('soundFile')
				}],
				"path": folder,
				"preload": true,
				"volume": 1.0
			});
		}

		opts.audio = ion.sound;
	},

	ajax: function(data, ajaxProperties) {
		var url = self.settings('path');

		if (self.settings('useIndexForXhr')) {
			url += '/index.php';
		}

		var token = self.settings('esToken');
		var data = $.extend({}, opts.ajaxData, data);

		// Insert the token
		data[token] = 1;

		var props = {
			"method": "post",
			"url": url,
			"data": data
		};

		if (ajaxProperties !== undefined) {
			props = $.extend({}, props, ajaxProperties);
		}

		var ajax = $.ajax(props);

		return ajax;
	},

	loading: function() {
		self.body().addClass('is-loading');
		self.contents().html('');
	},

	playSound: function() {
		if (!opts.audio) {
			return;
		}

		return opts.audio.play();
	},

	render: function(page, contents, callback) {

		// Reset all known classes on the body
		self.body().removeClass('is-loading');

		if (callback !== undefined) {
			callback.call(this, contents);
			return;
		}

		self.contents().html(contents);
	},

	scrollToTop: function() {
		$('.ckit-container__bd')
			.stop(true, false)
			.scrollTop(0);
	},

	scrollToBottom: function() {
		var wrapper = $('[data-ck-container-body-contents]');

		wrapper
			.stop(true, false)
			.animate({scrollTop: 200000}, 2000);

		// For non mobile, if manual scrolling is detected, immediately stop the process above. #295
		if (!opts.isIOS) {
			wrapper.on('scroll.ck.conversations', function() {
				wrapper.stop(true, true);
				wrapper.off('scroll.ck.conversations');
			});
		}
	},

	setCurrentPage: function(page) {
		opts.currentPage = page;

		self.setWindowStatus('open', page);

		// Set the current layout-xxx class to use
		self.body().removeClass('is-empty');
		self.window().switchClass('layout-' + page);
	},

	setCurrentTimeStamp: function(timestamp) {
		opts.currentTimeStamp = timestamp;
	},

	getCurrentTimeStamp: function() {
		return opts.currentTimeStamp;
	},

	addClass: function(className) {
		self.window().addClass(className);
	},

	removeClass: function(className) {
		self.window().removeClass(className);
	},

	setClass: function(className) {
		self.window().switchClass(className);
	},

	setTitle: function(title) {
		self.title().html(title);
	},

	settings: function(key) {
		return opts.settings[key];
	},

	setWindowStatus: function(state, page) {

		if (!self.settings('remember')) {
			return;
		}

		localStorage.setItem('chatWindow', state);
		localStorage.setItem('chatWindowPage', page);
	},

	getWindowStatus: function() {
		var state = {
			'window': 'close',
			'page': ''
		};

		if (!self.settings('remember')) {
			return state;
		}

		state.window = localStorage.getItem('chatWindow');
		state.page = localStorage.getItem('chatWindowPage');

		if (state.page == 'message' || state.page == 'register') {
			state.page = 'conversations';
		}

		return state;
	},

	updateCounter: function(count) {

		if (count > 0) {
			self.counter().html(count)
				.css('display', 'block');

			// integration with esmobile
			$('[data-conversation-notify-ckit]').html(count)
				.css('display', 'block');

			return;
		}

		self.counter().css('display', 'none');

		// integration with esmobile
		$('[data-conversation-notify-ckit]')
			.css('display', 'none');

	},

	updateMonitorWaitState: function(state) {
		self.monitoring.options.wait = state;
	},

	isOpen: function() {
		return self.button().hasClass('is-open');
	},


	isConversationOpen: function() {
		return self.window().hasClass('is-viewing-message');
	},

	isConversationListing: function() {
		return self.window().hasClass('layout-conversations');
	},

	open: function(start) {

		// clear counter on bubble
		self.updateCounter(0);
		monitoring.stop();

		// Fix legacy tab names by pluralizing it
		if (start == 'conversation' || start == 'contact' || start == 'setting') {
			start += 's';
		}

		// Render a tab in the background
		tabs.open(start);

		self.window().removeClass('is-hidden');
		self.button().addClass('is-open');

		monitoring.start();

		// Apply scroll locks on the browser
		setTimeout(function() {

			if (opts.isIOS) {
				self.scroll.lock();
				return;
			}

			self.element.css('position', 'relative');
		}, 200);
	},

	close: function() {
		self.setWindowStatus('close', '');

		self.window().addClass('is-hidden');
		self.button().removeClass('is-open');

		setTimeout(function(){

			if (opts.isIOS) {
				self.scroll.unlock();
				return;
			}

			self.element.css('position', '');

		}, 200);
	},

	// device platform
	platform: {

		isIOS: function() {
			var agent = navigator.userAgent.toLowerCase();
			var matches = agent.match(/(iphone|ipod|ipad)/);

			if (matches) {
				return true;
			}

			return false;
		},
		getIOSVersion: function(mainVersion) {
			var version = '';

			if (self.platform.isIOS()) {
				var agents = navigator.userAgent.split('OS ');
				if (agents.length > 1) {
					var tmp = agents[1].split(' ')[0];

					var subs = tmp.split('_');

					if (mainVersion) {
						version = subs[0];
					} else {
						version = subs.join('.')
					}
				}
			}

			return version;
		},
		isSafari: function() {
			var agent = navigator.userAgent.toLowerCase();
			return agent.indexOf("safari") > -1;
		},
		isFirefox: function() {
			var agent = navigator.userAgent.toLowerCase();
			return agent.indexOf("firefox") > -1;
		}

	},

	scroll: {
		original: {},

		lock: function() {

			window.bodyScrollTop = (typeof window.pageYOffset !== 'undefined') ? window.pageYOffset : (document.documentElement || document.body.parentNode || document.body).scrollTop;

			self.scroll.original = {
				'position': self.element.css('position'),
				'left': self.element.css('left'),
				'width': self.element.css('width'),
				'height': self.element.css('height'),
				'overflow': self.element.css('overflow'),
				'top': self.element.css('top')
			};

			self.element.css({
				'position': 'fixed',
				'left': 0,
				'width': '100%',
				'height': '100%',
				'overflow': 'hidden',
				'top': '-' + window.bodyScrollTop + 'px'
			});
		},

		unlock: function() {
			self.element.css(self.scroll.original);

			window.scrollTo(0, window.bodyScrollTop);
		}
	},

	"{self} onConverseKitUpdate": function(element, event, data, curtimestamp, timer) {

		// Ensure that we only play a new notification sound when there is new message
		var currentCount = parseInt(self.counter().html());

		if (!self.isOpen() && data > 0) {

			// Update the current counter
			self.updateCounter(data);
			self.updateMonitorWaitState(false);

			if (data > currentCount) {
				self.playSound();
			}
			return;
		}

		if (data > 0) {
			var namespace = "";
			var id = "";

			if (self.isConversationOpen()) {
				// viewing conversation. we need to get the new messages.
				namespace = 'getNotificationMessage';
				id = self.composer.options.conversationId;

			} else if (self.isConversationListing()) {
				// viewing conversations list.
				namespace = 'getNotificationConversations';
			}

			if (namespace) {
				self.ajax({
					"namespace": namespace,
					"timestamp": curtimestamp,
					"timer": timer,
					"id": id
				}).done(function(response) {
					if (response.lastmodif !== null) {
						tabs.trigger('onNewMessage', [response]);
					}

					self.updateMonitorWaitState(false);
				});
			}
		} else {
			self.updateMonitorWaitState(false);
		}
	},

	"{closeAlert} click": function(element, event) {
		self.alerts.hide();
	},

	"{closeButton} click": function(element, event) {
		self.close();
	},

	"{chat} click": function(element, event) {
		var recipientId = element.data('ck-chat');
		var conversationId = element.data('conversation-id');

		var start = self.settings('defaultTab');

		if (recipientId) {
			start = 'message:' + recipientId;
		} else if (conversationId) {
			start = 'conversations:' + conversationId;
		}

		self.open(start);
	},

	"{button} click": function(element, event) {
		var start = element.data('start');

		if (opts.userId == 0) {
			start = 'register';
		}

		if (self.isOpen()) {
			self.close();
			return;
		}

		self.open(start);
		return;
	}
}});

module.resolve();

});

});
EasySocial.module('ck/monitoring', function($) {

var module = this;

EasySocial.Controller('CK.Monitoring', {
	defaultOptions: {
		interval: 1,
		wait: false
	}
}, function(self, opts, base, parent) { return {

	init: function() {
		parent = self.parent;

		// Start monitoring
		self.start();
	},

	getInterval: function() {
		var interval = parent.settings('monitorInterval') * 1000;

		// Automatically increase the interval to 5 seconds when the tab is not active
		if (document.hidden) {
			interval = 5000;
		}

		return interval;
	},

	check: function() {

		setTimeout(function() {

			if (opts.wait) {
				return;
			}

			// update the wait so that we will wait
			// for the process to complete before we fire
			// another checking.
			opts.wait = true;

			// Determines the current state of the chat
			var state = parent.isOpen() ? 'open' : 'close';

			var currentTimeStamp = null;

			if (state == 'open') {
				currentTimeStamp = parent.getCurrentTimeStamp();
			}

			$.ajax({
				"url": window.es.rootUrl + '/plugins/system/conversekit/ajax.php',
				"method": "post",
				"data": {
					"state": state,
					"userId": parent.options.userId,
					"timestamp": currentTimeStamp
				}
			}).done(function(data) {

				var timestamp = null;
				var curtimestamp = null;

				if (data.indexOf('|') > -1) {
					var tmp = data.split('|');

					data = parseInt(tmp[0]);
					timestamp = tmp[1];
					curtimestamp = tmp[2];

					parent.setCurrentTimeStamp(timestamp);
				} else {
					data = parseInt(data);
				}

				// Trigger result
				// Also, the wait state will be updated by onConverseKitUpdate
				parent.trigger('onConverseKitUpdate', [data, curtimestamp, self.getInterval()]);

				// Start the checking again
				self.start();
			});

		}, self.getInterval());
	},

	start: function() {
		opts.state = setTimeout(self.check, self.getInterval());
	},

	stop: function() {
		clearTimeout(opts.state);
	}

}});

module.resolve();

});
EasySocial.module('ck/tabs', function($) {

var module = this;

EasySocial.require()
.script('ck/settings', 'ck/conversations', 'ck/contacts')
.done(function($) {

EasySocial.Controller('CK.Tabs', {
	defaultOptions: {
		activeTab: null,
		"{container}": "[data-ck-container]",
		"{tabs}": "[data-ck-footer-tabs]",
		"{tab}": "[data-ck-footer-tabs] [data-ck-tab]"
	}
}, function(self, opts, base, ck, conversations) { return {

	init: function() {
		ck = self.parent;

		// TODO: Find better ways to implement them
		self.container().implement(EasySocial.Controller.CK.Settings, {
			"{parent}": self,
			"{ck}": ck
		});

		self.conversations = conversations = self.container().addController(EasySocial.Controller.CK.Conversations, {
												"{parent}": self,
												"{ck}": ck
											});

		self.contacts = contacts = self.container().implement(EasySocial.Controller.CK.Contacts, {
										"{parent}": self,
										"{ck}": ck
									});
	},

	// Called from extended classes
	render: function(namespace, data, callback) {

		var options = {
			'namespace': namespace
		};

		// console.log(namespace);

		options = $.extend({}, options, data);

		ck.loading();

		ck.ajax(options).done(function(contents) {
			ck.render(opts.activeTab, contents, callback);
		});
	},

	open: function(tab) {
		var namespace = tab.split(':');

		// Rendering conversation
		if (namespace.length > 1) {
			if (namespace[0] == 'message') {
				conversations.renderConversationWithUserId(namespace[1]);
			}

			if (namespace[0] == 'conversations') {
				conversations.renderConversation(namespace[1]);
			}

			return;
		}

		opts.activeTab = tab;
		ck.setCurrentPage(tab);

		var element = self.tabs().find('[data-ck-tab=' + tab + ']');
		var title = element.data('title');

		// Add active state on the current tab
		self.tab().removeClass('is-active');
		element.addClass('is-active');

		if (title !== undefined) {
			ck.setTitle(title);
		}

		ck.body().removeClass('has-filters has-actions');
		self.container().trigger('onRenderTab', [opts.activeTab]);
	},

	"{tab} click": function(element, event) {
		var tab = element.data('ck-tab');

		self.open(tab);
	}
}});

});

module.resolve();

});
EasySocial.module('ck/settings', function($) {

var module = this;

EasySocial.Controller('CK.Settings', {
	defaultOptions: {
		"{option}": "[data-settings-option]",
		"{form}": "[data-settings-form]",

		"{sound}": "[data-settings-sound]",
		"{submit}": "[data-settings-submit]",
		"{darkmode}": "[data-settings-darkmode]"
	}
}, function(self, opts, base, parent, ck) { return {
	init: function() {
		parent = self.parent;
		ck = self.ck;
	},

	"{option} change": function(element, event) {
		var checked = element.is(':checked');
		var ckWrapper = $('#ck');

		// Darkmode enabled
		if (self.darkmode().is(':checked')) {
			ckWrapper
				.attr('data-ck-theme', 'dark');
		} else {
			ckWrapper
				.attr('data-ck-theme', 'light');
		}

		ck.ajax({
			'namespace': 'saveSettings',
			'sound': self.sound().is(':checked') ? 1 : 0,
			'submit': self.submit().is(':checked') ? 1 : 0,
			'darkmode': self.darkmode().is(':checked') ? 1 : 0
		});
	},

	"{self} onRenderTab": function(element, event, tab) {

		if (tab != 'settings') {
			return;
		}

		// Render contents
		parent.render('renderSettings');
	}
}});

module.resolve();

});EasySocial.module('ck/conversations', function($) {

var module = this;

EasySocial.Controller('CK.Conversations', {
	defaultOptions: {
		selectedItems: [],
		conversationId: null,
		recipientId: null,
		limitstart: 0,
		newMessageCounts: 0,

		"{checkAll}": "[data-ck-checkall]",
		"{massEdit}": "[data-ck-mass-edit]",
		"{massEditDone}": "[data-ck-mass-edit-done]",

		"{conversationItem}": "[data-ck-conversation-item]",
		"{checkboxItems}": "[data-checkbox] input:checkbox",

		"{compose}": "[data-ck-compose]",
		"{item}": "[data-ck-conversation-item]",
		"{toggle}": "[data-inner-toggle]",

		// load more
		"{loadMoreButton}": "[data-ck-loadmore-conversations]",
		"{loadMoreMessageButton}": "[data-ck-loadmore-messages]",

		// Giphy
		"{giphyButton}" : "[data-ck-composer-giphy]",
		"{giphyItem}" : "[data-ck-composer-giphy-item]",

		// Single chat
		"{conversationHeader}": "[data-ck-container-header-message]",
		"{backButton}": "[data-ck-back]",
		"{addParticipant}": "[data-ck-add-participants]",
		"{viewParticipants}": "[data-ck-view-participants]",
		"{insertParticipant}": "[data-ck-insert-participant]",
		"{leave}": "[data-ck-leave-conversations]",
		"{updateTitle}": "[data-ck-change-title]",

		// Title changes
		"{title}": "[data-ck-title]",
		"{titleInput}": "[data-ck-title-input]",
		"{saveTitleButton}": "[data-ck-title-save]",
		"{cancelSaveTitle}": "[data-ck-title-cancel]",

		// Mass actions
		"{actions}": "[data-ck-actions]",
		"{archive}": "[data-ck-archive]",
		"{markRead}": "[data-ck-mark-read]",
		"{delete}": "[data-ck-delete]",
		"{confirmDelete}": "[data-ck-delete-confirm]",
		"{cancelDelete}": "[data-ck-delete-cancel]",

		// Attachments
		"{confirmDeleteAttachment}": "[data-ck-attachment-delete]",
		"{attachmentWrapper}": "[data-ck-attachment-item-wrapper]",
		"{deleteAttachment}": "[data-ck-attachment-item-wrapper] [data-ck-confirm]",
		"{cancelDeleteAttachment}": "[data-ck-attachment-item-wrapper] [data-ck-cancel]"
	}
}, function(self, opts, base, parent, ck) { return {
	init: function() {
		parent = self.parent;
		ck = self.ck;
	},

	hideActions: function() {
		parent.parent.body().removeClass('has-actions');
	},

	showActions: function() {
		// There is no mass actions to add participant in conversations listing
		self.actions().find('[data-ck-add]').addClass('hidden');

		parent.parent.body().addClass('has-actions');
	},

	uncheckAll: function() {
		self.checkboxItems()
			.prop('checked', false)
			.trigger('change');
	},

	renderMessageWindow: function(conversation) {

		if (conversation.canReply) {
			ck.setClass('can-reply');
		} else {
			ck.removeClass('can-reply');
		}

		if (conversation.isNew) {
			ck.setClass('is-new');
		} else {
			ck.removeClass('is-new');
		}

		if (conversation.isGroup) {
			ck.addClass('is-group');
		} else {
			ck.removeClass('is-group');
		}

		// Update the header message
		self.conversationHeader().html(conversation.header);

		// Set the contents
		ck.render('message', conversation.contents);

		// Ensure that the chat always scrolls to the bottom
		ck.scrollToBottom();
	},

	// Renders a conversation with a given user id.
	// ConverseKit will figure out the conversation id.
	renderConversationWithUserId: function(userId) {
		ck.loading();

		ck.setClass('is-viewing-message');
		ck.removeClass('can-reply');

		// Ensure that the contents are always empty so we don't get jumpy effects
		self.conversationHeader().html('');
		ck.setCurrentPage('message');

		ck.ajax({
			"namespace": "renderMessageWithUserId",
			"userId": userId
		}).done(function(conversation) {

			self.renderMessageWindow(conversation);

			// If this is not a new conversation, we should set the conversation id
			opts.conversationId = null;

			if (conversation.id !== undefined) {
				opts.conversationId = conversation.id;
			}

			ck.composer.trigger('onRenderMessage', [opts.conversationId, userId]);
		});
	},

	renderConversation: function(id) {
		ck.loading();

		ck.removeClass('can-reply');
		ck.setClass('is-viewing-message');
		opts.conversationId = id;

		// Ensure that the contents are always empty so we don't get jumpy effects
		self.conversationHeader().html('');
		ck.setCurrentPage('message');

		opts.limitstart = 0;

		ck.ajax({
			"namespace": "renderMessage",
			"id": id,
			"limit": ck.settings('listLimit')
		}).done(function(conversation) {

			self.renderMessageWindow(conversation);

			if (conversation.timestamp != undefined) {
				ck.setCurrentTimeStamp(conversation.timestamp);
			}

			ck.composer.trigger('onRenderMessage', id);
		});
	},

	renderParticipants: function(conversationId) {
		ck.loading();
		ck.setClass('is-viewing-participants');

		parent.contacts.trigger('onViewParticipant', opts.conversationId);

		ck.ajax({
			"namespace": "renderParticipants",
			"id": conversationId,
			"limit": ck.settings('contactLimit')
		}).done(function(contents) {
			ck.render('participants', contents);
		});
	},

	removeSelectedItems: function() {
		$(opts.selectedItems).each(function(i, id) {
			var item = self.conversationItem('[data-id=' + id +']');
			item.remove();
		});

		opts.selectedItems = [];

		// When items are removed, we also need to ensure that everything is unchecked.
		self.uncheckAll();

		// If there is nothing left, we need to add the empty state
		if (self.conversationItem().length <= 0) {
			ck.window().addClass('is-empty');
			ck.window().removeClass('is-editing');
		}
	},

	"{cancelDeleteAttachment} click": function(element, event) {
		var wrapper = element.parents(self.attachmentWrapper.selector);

		wrapper.removeClass('confirm-remove');
	},

	"{deleteAttachment} click": function(element, event) {
		var wrapper = element.parents(self.attachmentWrapper.selector);
		var id = wrapper.data('id');

		ck.ajax({
			'namespace': 'deleteAttachment',
			'fileId': id
		}).done(function() {
			wrapper.remove();
		});
	},

	"{confirmDeleteAttachment} click": function(element, event) {
		var wrapper = element.parents('[data-ck-attachment-item-wrapper]');

		wrapper.addClass('confirm-remove');
	},

	//
	// Custom conversation title
	//
	saveTitle: function(title) {
		ck.removeClass('is-editing');

		ck.ajax({
			"namespace": "updateTitle",
			"conversationId": opts.conversationId,
			"title": title
		}).done(function(response) {
			self.title().html(response.title);

			// Update the original title
			self.titleInput().attr('data-original', response.title);
		});
	},

	"{titleInput} keyup": function(element, event) {
		var title = $.trim(element.val());
		var original = element.data('original');

		if (event.keyCode == '27') {
			element.val(original);
			self.cancelSaveTitle().click();
			return;
		}

		if (event.keyCode == '13' && title != '') {
			self.saveTitle(title);
		}
	},

	"{saveTitleButton} click": function(element, event) {
		var title = self.titleInput().val();

		self.saveTitle(title);
	},

	"{cancelSaveTitle} click": function(element, event) {
		ck.removeClass('is-editing');
	},

	"{updateTitle} click": function(element, event) {
		ck.addClass('is-editing');
	},

	//
	// Participants for group conversations
	//
	"{viewParticipants} click": function(element, event) {
		self.renderParticipants(opts.conversationId);
	},

	"{addParticipant} click": function(element, event) {
		ck.setCurrentPage('add-participants');
		ck.setClass('is-viewing');

		parent.contacts.trigger('onAddParticipant', opts.conversationId);

		// Render the contacts listing for the user
		parent.render('renderContacts', {
			"addParticipant": true,
			"conversationId": opts.conversationId,
			"limit": ck.settings('contactLimit')
		});
	},

	"{insertParticipant} click": function(element, event) {
		event.stopPropagation();

		var contactId = element.data('id');

		ck.ajax({
			"namespace": "addParticipants",
			"conversationId": opts.conversationId,
			"participantId": contactId
		}).done(function(response) {

			// Re-render the conversation
			self.renderConversation(opts.conversationId);
		});
	},

	"{leave} click": function(element, event) {

		ck.ajax({
			"namespace": "leave",
			"conversationId": opts.conversationId
		}).done(function() {

			// Reset the conversation id
			opts.conversationId = null;

			// Render the contacts
			parent.open('conversations');
		});
	},

	"{backButton} click": function(element, event) {

		if (ck.window().hasClass('is-viewing-participants')) {
			self.renderConversation(opts.conversationId);
			return false;
		}

		if (ck.window().hasClass('layout-add-participants')) {
			self.renderConversation(opts.conversationId);

			return false;
		}

		// Always go back to the conversation listing
		ck.open('conversations');
	},

	"{markRead} click": function(element, event) {
		if (opts.selectedItems.length <= 0) {
			return;
		}

		// Fire an ajax call to archive the conversations
		ck.ajax({
			"namespace": "markAsRead",
			"id": opts.selectedItems
		}).done(function(data) {

			self.uncheckAll();
			self.hideActions();

			ck.alerts.success(data.message);
		});
	},

	"{archive} click": function(element, event) {

		if (opts.selectedItems.length <= 0) {
			return;
		}

		// Fire an ajax call to archive the conversations
		ck.ajax({
			"namespace": "archive",
			"id": opts.selectedItems
		}).done(function() {
			self.removeSelectedItems();
		});
	},

	"{delete} click": function(element, event) {
		self.actions().addClass('confirm-remove');
	},

	"{confirmDelete} click": function(element, event) {

		self.actions().removeClass('confirm-remove');

		// Perform an ajax call to delete these conversations
		ck.ajax({
			'namespace': 'deleteConversations',
			'id': opts.selectedItems
		}).done(function(response) {

			self.removeSelectedItems();

			ck.alerts.success(response.message);
		});
	},

	"{cancelDelete} click": function(element, event) {
		self.actions().removeClass('confirm-remove');
	},

	"{checkboxItems} change": function(element, event) {
		var checked = element.is(':checked');
		var id = element.val();

		if (checked) {
			self.showActions();
			opts.selectedItems.push(id);

			return;
		}

		opts.selectedItems = $.grep(opts.selectedItems, function(value) {
								return value != id;
							});

		if (opts.selectedItems.length <= 0) {
			self.hideActions();
		}
	},

	"{checkAll} change": function(element, event) {

		var checked = element.is(':checked');

		if (checked) {
			self.checkboxItems().not(element)
				.prop('checked', checked)
				.trigger('change');

			return;
		}

		self.checkboxItems().not(element)
			.prop('checked', false)
			.trigger('change');
	},

	"{massEditDone} click": function() {
		self.uncheckAll();

		ck.setClass('is-viewing');
	},

	"{massEdit} click": function() {
		ck.setClass('is-editing');
	},

	"{compose} click": function(element, event) {

		// Render the contacts tab so the user can start a new chat
		parent.open('contacts');
	},

	"{item} click": function(element, event) {
		var target = $(event.target);

		// In editing mode, clicking on the item should check the checkbox
		if (ck.window().hasClass('is-editing')) {
			var checked = element.find(self.checkboxItems.selector).is(':checked');

			element.find(self.checkboxItems.selector)
				.prop('checked', !checked)
				.trigger('change');

			return false;
		}

		// Clicking on the dropdown should not fire rendering of chat
		if (target.data('bs-toggle') == 'dropdown' || target.parents('[data-chat-dropdown]').length > 0) {
			return;
		}

		var id = element.data('id');

		self.renderConversation(id);
	},

	"{parent} onNewMessage": function(element, event, response) {

		if (ck.isConversationOpen()) {
			// Append new message to the content
			ck.contents().append(response.contents);
			ck.scrollToBottom();
		} else if (ck.isConversationListing()) {

			if (response.data != undefined && response.total > 0) {

				$.each(response.data, function (i, item) {
					var id = item.conversationId;
					var convItem = $('[data-ck-conversation-item][data-id="' + id + '"]');
					var notify = convItem.find("[data-chat-notify]");
					var currentCount = parseInt(notify.html());

					notify
						.html(item.count)
							.addClass('t-block')
								.removeClass('t-hidden');

					convItem.find("[data-ck-conversation-preview]").html(item.preview);

					convItem.find('[data-ck-conversation-date]').html(item.dateLapse);
				});
			}
		}
	},


	"{self} onRenderTab": function(element, event, tab) {
		if (tab !== 'conversations') {
			return;
		}

		// for pagination
		var options = {
			"limit": ck.settings('listLimit'),
			"sorting": ck.settings('conversationSorting'),
			"ordering": ck.settings('conversationOrdering'),
		};

		parent.render('renderConversations', options, function(output) {
			var ck = this;

			if (output.empty !== undefined) {
				ck.window().addClass('is-empty');
				return;
			}

			ck.contents().html(output.contents);

			ck.setCurrentTimeStamp(output.timestamp);
		});

		opts.limitstart = 0;
		opts.newMessageCounts = 0;

		// Reset the conversation id
		self.conversationId = null;
		ck.setClass('is-viewing');
	},

	// loadmore
	loadmore: function(data, button) {

		if (opts.limitstart == 0) {
			opts.limitstart = data.limit;
		}

		if (opts.newMessageCounts > 0) {
			opts.limitstart += opts.newMessageCounts;
		}

		var options = {
			'limitstart': opts.limitstart
		};

		options = $.extend({}, options, data);

		// Apply loading class on the button instead
		button.addClass('is-loading');

		ck.ajax(options).done(function(data) {
			button.removeClass('is-loading');
			// reset new message count as its no longer needed.
			opts.newMessageCounts = 0;

			if (data.contents != '') {

				if (options.namespace == 'loadMoreConversations') {
					$(data.contents).insertBefore(button);
				} else {
					// update contents
					$(data.contents).insertAfter(button);
				}

				if (data.nextlimit < 0) {
					button.hide();
				} else {
					opts.limitstart = data.nextlimit;
				}
			}

		});

	},

	"{loadMoreButton} click": function (element, event) {

		// this count is not applicable to conversations pagination.
		opts.newMessageCounts = 0;

		// for pagination
		var options = {
			'namespace': 'loadMoreConversations',
			"limit": ck.settings('listLimit')
		};

		self.loadmore(options, element);
	},

	"{loadMoreMessageButton} click": function (element, event) {

		// for pagination
		var options = {
			'namespace': 'loadMoreMessages',
			"limit": ck.settings('listLimit'),
			"id": opts.conversationId
		};

		self.loadmore(options, element);
	}


}});

module.resolve();

});
EasySocial.module('ck/contacts', function($) {

var module = this;

EasySocial.Controller('CK.Contacts', {
	defaultOptions: {

		filters: null,
		limitstart: 0,

		"{item}": "[data-ck-contact-item]",
		"{filter}": "[data-ck-filter]",
		"{searchInput}": "[data-ck-search-input]",
		"{clear}": "[data-ck-search-clear]",
		"{loadMoreButton}": "[data-ck-loadmore]"
	}
}, function(self, opts, base, parent, ck) { return {
	init: function() {
		parent = self.parent;
		ck = self.ck;
	},

	clearSearch: function() {
		self.searchInput().val('');
		ck.setClass('is-viewing');
	},

	isAddingParticipant: function() {

		// Adding new participant
		if (ck.window().hasClass('layout-add-participants') && opts.conversationId) {
			return true;
		}

		return false;
	},

	isViewingParticipant: function() {
		return ck.window().hasClass('is-viewing-participants');
	},

	search: function(text) {
		ck.setClass('is-searching');

		var options = {
			"search": text,
			"limit": ck.settings('contactLimit')
		};

		if (self.isAddingParticipant()) {
			options['addParticipant'] = true;
			options['conversationId'] = opts.conversationId;
		}

		// keep this for pagination use.
		opts.filters = options;

		parent.render('searchContacts', options);
	},

	renderContacts: function(filter) {
		var filter = filter === undefined ? 'all' : filter;
		var options = {
			"filter": filter,
			"limit": ck.settings('contactLimit')
		};

		// Adding new participant
		if (self.isAddingParticipant()) {
			options['addParticipant'] = true;
			options['conversationId'] = opts.conversationId;
		}

		// console.log(options);

		// keep this for pagination use.
		opts.filters = options;

		parent.render('renderContacts', options);

		ck.setClass('is-viewing');
	},

	"{self} onAddParticipant": function(element, event, conversationId) {
		opts.conversationId = conversationId;

		var options = {
			"addParticipant": true,
			"conversationId": opts.conversationId,
			"limit": ck.settings('contactLimit')
		};

		// keep this for pagination use.
		opts.filters = options;

		// reset limitstart
		opts.limitstart = 0;

	},

	"{self} onViewParticipant": function(element, event, conversationId) {
		opts.conversationId = conversationId;

		var options = {
			"filter": "participants",
			"id": opts.conversationId,
			"limit": ck.settings('contactLimit')
		};

		// keep this for pagination use.
		opts.filters = options;

		// reset limitstart
		opts.limitstart = 0;

	},

	"{item} click": function(element, event) {
		var id = element.data('id');
		var namespace = 'message:' + id;

		// When adding participant, clicking on the user shouldn't do anything
		if (self.isAddingParticipant()) {
			return false;
		}

		// In viewing participants mode, clicking on the item should not open new chat
		if (self.isViewingParticipant()) {
			return false;
		}

		parent.open(namespace);
	},

	// loadmore
	loadmore: function(data) {

		if (opts.limitstart == 0) {
			opts.limitstart = data.limit;
		}

		var options = {
			'namespace': 'loadMoreContacts',
			'limitstart': opts.limitstart
		};

		options = $.extend({}, options, data);

		// Apply loading class on the button instead
		self.loadMoreButton().addClass('is-loading');

		ck.ajax(options).done(function(data) {
			self.loadMoreButton().removeClass('is-loading');

			if (data.contents != '') {

				// update contents
				$(data.contents).insertBefore(self.loadMoreButton());

				if (data.nextlimit < 0) {
					self.loadMoreButton().hide();
				} else {
					opts.limitstart = data.nextlimit;
				}

			}

		});

	},

	"{loadMoreButton} click": function() {
		// keep this for pagination use.
		var filters = opts.filters;

		self.loadmore(filters);
	},


	"{searchInput} keyup": $.debounce(function(element, event) {

		var text = element.val();

		// If the user hits the enter key, do not allow browser to propagate the event
		if (event.keyCode == '13') {
			event.stopPropagation();
			event.preventDefault();

			return false;
		}

		if ($.trim(text) == '') {
			self.renderContacts();
			return;
		}

		self.search(text);

	}, 300),

	"{clear} click": function(element, event) {
		var search = $.trim(self.searchInput().val());

		if (search == '') {
			return;
		}

		self.clearSearch();
		self.renderContacts();
	},

	"{self} onRenderTab": function(element, event, tab) {
		if (tab !== 'contacts') {
			return;
		}
		self.clearSearch();
		self.renderContacts();

		opts.limitstart = 0;

		// Set initial filter to all
		self.filter('[data-ck-filter=all]').activateClass('active');
	},

	"{filter} click": function(element, event) {
		var filter = element.data('ck-filter');

		opts.limitstart = 0;

		self.filter('[data-ck-filter=' + filter +']').activateClass('active');
		self.renderContacts(filter);
	}
}});

module.resolve();

});
EasySocial.module('ck/composer', function($) {

var module = this;

EasySocial.Controller('CK.Composer', {
	defaultOptions: {
		conversationId: null,
		recipientId: null,
		templates: [],

		"{composer}": "[data-ck-composer]",
		"{composerForm}": "[data-ck-composer-form]",
		"{dropzoneWrapper}": "[data-ck-composer-dropzone]",

		"{textarea}": "[data-ck-composer-textarea]",
		"{submit}": "[data-reply-submit]",

		"{contents}": "[data-ck-container-body-contents]",

		// Attachments
		"{attachment}": "[data-ck-composer-attachments]",
		"{attachmentInput}": "[data-ck-composer-attachments-input]",
		"{attachmentList}": "[data-ck-composer-attachments-list]",
		"{attachmentItem}": "[data-ck-attachment-item]",
		"{attachmentTemplate}": "[data-ck-attachment-template]",
		"{removeAttachment}": "[data-ck-attachment-delete]",

		// Giphy
		"{giphyButton}" : "[data-ck-composer-giphy]",
		"{giphyBrowser}" : "[data-ck-giphy-browser]",
		"{giphyItem}" : "[data-ck-composer-giphy-item]",
		"{giphySearch}" : "[data-ck-giphy-search]",
		"{giphyList}" : "[data-ck-giphy-list]",
		"{giphyTrending}" : "[data-ck-giphy-trending]",
		"{giphyLoading}" : "[data-ck-giphy-loading]",

		// Location
		"{location}": "[data-ck-composer-location]",
		"{locationWrapper}": "[data-ck-location-wrapper]",
		"{locationLatitude}": "[data-ck-composer-location-latitude]",
		"{locationLongitude}": "[data-ck-composer-location-longitude]",
		"{locationAddress}": "[data-ck-composer-location-address]",
		"{locationTitle}": "[data-ck-composer-location-title]",
		"{removeLocation}": "[data-ck-location-remove]",

		// Emoji
		"{emoji}": "[data-ck-composer-emoji]",
		"{emojiItem}": "[data-ck-composer-emoji-item]",

		// warnign message wrapper
		"{warningWrapper}" : "[data-ck-warning-wrapper]"
	}
}, function(self, opts, base, parent, ck) { return {
	init: function() {
		ck = parent = self.parent;

		// Initialize attachment template
		if (self.attachmentList().length > 0) {
			opts.templates['attachment'] = $('<div>').html(self.attachmentTemplate()[0]);

			// Remove the template file
			self.attachmentTemplate().remove();
		}
	},

	getTemplate: function(type) {
		var tpl = opts.templates[type];

		var template = tpl.clone();

		return $(template.html());
	},

	insertText: function(text) {
		var val = self.textarea().val();
		val += ' ' + text;

		self.textarea().val(val);
	},

	reset: function() {
		self.textarea().val('');

		self.attachmentList().empty();

		self.clearLocation();
	},

	"{giphySearch} keyup": $.debounce(function(el) {
		// Get the search query
		var query = el.val();

		var loading = self.giphyLoading();
		var giphyList = self.giphyList();
		var trending = self.giphyTrending();

		giphyList.addClass('t-hidden');
		loading.addClass('is-active');

		trending.removeClass('t-hidden');

		if (query != '') {
			trending.addClass('t-hidden');
		}

		var options = {
			'namespace': 'searchGiphy',
			'query': query
		};

		self.loadGiphy(options);
	}, 300),

	isGiphyBrowserHidden: function() {
		return self.giphyBrowser().hasClass('t-hidden');
	},

	hideGiphyBrowser: function() {
		self.giphyBrowser().addClass('t-hidden');
	},

	showGiphyBrowser: function() {
		self.giphyBrowser().removeClass('t-hidden');
	},

	_isGiphyLoaded: false,

	loadGiphy: function(options) {
		var loading = self.giphyLoading();
		var giphyList = self.giphyList();

		parent.ajax(options)
			.done(function(data) {
				loading.removeClass('is-active');
				giphyList.removeClass('t-hidden');

				giphyList.html(data.contents);
			});
	},

	initializeGiphyBrowser: function() {
		// We only initialize once
		if (self._isGiphyLoaded) {
			return;
		}

		self._isGiphyLoaded = true;

		var loading = self.giphyLoading();
		loading.addClass('is-active');

		var options = {
			'namespace': 'searchGiphy',
			'query': ''
		};

		self.loadGiphy(options);
	},

	"{giphyButton} click": function(el, event) {
		self.initializeGiphyBrowser();

		if (self.isGiphyBrowserHidden()) {
			self.showGiphyBrowser();

			return;
		}

		self.hideGiphyBrowser();
	},

	"{giphyItem} click": function(el, event) {
		imageUrl = el.data('original');

		self.insertGiphy(imageUrl);

		// Close the giphy browser
		self.giphyBrowser().addClass('t-hidden')
	},

	insertGiphy: function(url) {
		self.send(imageUrl, true);
	},

	send: function(message, isGiphy) {
		var type = isGiphy ? 'giphy' : 'message';
		var attachments = self.getAttachments();
		var location = self.getLocation();
		var namespace = opts.conversationId ? 'reply' : 'create';

		var options = {
			'namespace': namespace,
			'uid': opts.conversationId,
			'recipientId': opts.recipientId,
			'message': message,
			'type': type
		};

		// Do not send attachement or location when sending a giphy
		if (!isGiphy) {
			if (attachments) {
				options['upload-id'] = attachments;
			}

			if (location) {
				options = $.extend({}, options, location);
			}
		}

		self.hideWarning();

		parent.ajax(options)
			.done(function(output) {

				if (output.code != undefined && output.message != undefined) {
					// has error.
					self.showWarning(output);
					return;
				}

				// Regardless if this is a new conversation or reply, we always set the conversation id.
				opts.conversationId = output.id;

				// When a message is sent, ensure that is-new is removed
				ck.removeClass('is-new');

				if (isGiphy) {
					var item = $(output.contents);

					// Add loader to the giphy image to tell the users that the image hasn't finish load
					item.find('[data-ck-giphy-item-preview-placeholder]').addClass('is-loading');

					item.find('[data-ck-giphy-item-preview]')
						.off('load')
							.on('load', function() {
								item.find('[data-ck-giphy-item-preview-placeholder]').removeClass('is-loading');

								// Need to scroll to bottom again after the image finished loaded
								parent.scrollToBottom();
							});

					// Append message to the content
					parent.contents().append(item);
				} else {
					// Append message to the content
					parent.contents().append(output.contents);
				}

				parent.scrollToBottom();

				// we need to increment the limitstart from conversations controller
				// so that the same message will not be retrieve again.
				parent.tabs.conversations.options.newMessageCounts += 1;

				// update other controller the options.conversationId
				if (namespace == 'create') {
					parent.tabs.conversations.options.conversationId = output.id;
				}

				// If is sent by giphy, we do not need to reset the form
				if (!isGiphy) {
					// Reset the form before the submission happens
					self.reset();
				}
			});
	},

	showWarning: function(err) {
		self.warningWrapper().html(err.message);
		self.warningWrapper().show();
	},

	hideWarning: function() {
		self.warningWrapper().html('');
		self.warningWrapper().hide();
	},

	getAttachments: function() {
		var items = [];

		self.attachmentItem().each(function(i, item) {
			var id = $(item).data('id');

			items.push(id);
		});

		return items;
	},

	getLocation: function() {
		var location = {
			"latitude": self.locationLatitude().val(),
			"longitude": self.locationLongitude().val(),
			"address": self.locationAddress().val()
		};

		if (!location.latitude && !location.longitude) {
			return false;
		}

		return location;
	},

	//
	// Attachments
	//
	insertAttachment: function(attachments) {
		$.each(attachments, function(i, attachment) {
			var form = new FormData();
			form.append('file', attachment, attachment.name, attachment.size, attachment.type);
			form.append('namespace', 'uploadAttachment');
			form.append(ck.settings('esToken'), 1);
			form.append('conversekit', 1);
			form.append('format', 'json');
			form.append('option', ck.options.component);

			self.composerForm().addClass('has-file');

			var template = self.getTemplate('attachment');
			template.find('[data-name]').html(attachment.name);
			template.addClass('is-uploading');

			self.attachmentList().append(template);

			ck.ajax({}, {
				data: form,
				contentType: false,
				processData: false,
				enctype: 'multipart/form-data'
			}).done(function(data) {
				template
					.addClass('is-success')
					.removeClass('is-uploading');

				template.attr('data-id', data.id);

				// Show the attachment error if there is
				if (data.code != undefined && data.message != undefined) {
					self.showWarning(data);
					return;
				}
			}).error(function(data) {
				template
					.addClass('is-error')
					.removeClass('is-uploading');
			});
		});
	},

	"{attachment} click": function(element, event) {
		// Trigger the click event on the file input
		self.attachmentInput().click();

		// Hide the giphy browser if it is enabled as it covers the whole form
		if (!self.isGiphyBrowserHidden()) {
			self.hideGiphyBrowser();
		}
	},

	"{attachmentInput} change": function(element, event) {
		var files = element[0].files;

		self.insertAttachment(files);
	},

	"{removeAttachment} click": function(element, event) {
		var item = element.parents(self.attachmentItem.selector);

		item.remove();
	},

	dropzone: {
		activated: false,

		show: function() {
			self.dropzone.activated = true;
			self.composerForm().addClass('is-dragover');
		},

		hide: function() {
			self.dropzone.activated = false;
			self.composerForm().removeClass('is-dragover');
		}
	},

	"{composerForm} dragover": function(element, event) {
		event.preventDefault();

		// Due to the way drag event is triggered, we need to only bind the necessary event only when the drag is activated
		if (!self.dropzone.activated) {
			self.dropzone.show();

			// Set a slight delay due to ondrag fire events every few miliseconds.
			setTimeout(function() {
				self.composerForm().on('dragleave', function() {
					self.dropzone.hide();

					self.composerForm().off('dragleave');
				});
			}, 25);
		}
	},

	"{composerForm} drop": function(element, event) {
		event.preventDefault();
		event.stopPropagation();

		self.dropzone.hide();

		return false;
	},

	"{self} dropFiles": function(element, event, files) {
		self.insertAttachment(files);

		self.dropzone.hide();
	},

	//
	// Emoji
	//
	"{emoji} click": function(element, event) {
		self.composer().toggleClass('has-icons');

		// Hide the giphy browser if it is enabled as it covers the whole form
		if (!self.isGiphyBrowserHidden()) {
			self.hideGiphyBrowser();
		}
	},

	"{emojiItem} click": function(element, event) {
		var command = element.data('command');

		self.insertText(command);
		self.emoji().click();
	},

	//
	// Location
	//
	clearLocation: function() {
		self.locationLatitude().val('');
		self.locationLongitude().val('');
		self.locationAddress().val('');

		self.locationWrapper().removeClass('is-success is-error');
		self.composerForm().removeClass('has-location is-loading');
	},

	geolocate: function(position) {
		var latitude = position.coords.latitude;
		var longitude = position.coords.longitude;
		var url = "https://maps.googleapis.com/maps/api/geocode/json?latlng=" + latitude + "," + longitude + "&key=" + ck.settings('googleapi');

		// Query google's api
		ck.ajax({}, {
			"url": url,
			"dataType": "json",
			"type": "get"
		}).done(function(data) {

			self.composerForm().removeClass('is-loading');

			if (data.results[0] === undefined) {
				self.locationWrapper().addClass('is-error');

				return;
			}

			self.locationWrapper().addClass('is-success');

			var address = data.results[0].formatted_address;

			self.locationTitle().html(address);
			self.locationLatitude().val(latitude);
			self.locationLongitude().val(longitude);
			self.locationAddress().val(address);
		}).error(function() {
			console.log('error', arguments);
		});
	},

	locateosm: function() {
		self.getUserCoords()
			.then(function(coords){
				EasySocial.ajax('site/controllers/location/getLocations', {
					latitude: coords.latitude,
					longitude: coords.longitude,
				})
				.done(function(results) {
					self.composerForm().removeClass('is-loading');
					result = results[0];

					if (result === undefined) {
						self.locationWrapper().addClass('is-error');

						return;
					}

					self.locationWrapper().addClass('is-success');

					var address = result.formatted_address;

					self.locationTitle().html(address);
					self.locationLatitude().val(coords.latitude);
					self.locationLongitude().val(coords.longitude);
					self.locationAddress().val(address);
				});
			})
			.fail(function(message){
				console.log('error', message);
			});

	},

	getUserCoordsByIP: function() {
		var request =
			$.getJSON("//www.telize.com/geoip?callback=?")
				.then(function(data) {
					return {latitude: data.latitude, longitude: data.longitude};
				});
	},

	userCoords: null,

	getUserCoords: function() {
		var task = $.Deferred();
		var userCoords = self.userCoords;

		// If we have user lat & lng, automatically resolve this.
		if (userCoords) {
			task.resolve(userCoords);
			return task;
		}

		if (navigator.geolocation) {
			navigator.geolocation.getCurrentPosition(

				// If successful
				function(position) {
					var coords = position.coords;

					task.resolve({
						latitude: coords.latitude,
						longitude: coords.longitude
					});
				},

				// If failed, get user coords by IP.
				function() {
					self.getUserCoordsByIP()
						.done(task.resolve)
						.fail(task.reject);
				}

			);
		} else {
			task = self.getUserCoordsByIP();
		}

		return task.done(function(coords){
			self.userCoords = coords;
		});
	},

	"{location} click": function(element, event) {
		// Hide the giphy browser if it is enabled as it covers the whole form
		if (!self.isGiphyBrowserHidden()) {
			self.hideGiphyBrowser();
		}

		self.composerForm().addClass('has-location is-loading');

		if (ck.settings('mapProvider') == 'osm') {
			self.locateosm();
		} else {
			if (navigator.geolocation) {
				navigator.geolocation.getCurrentPosition(self.geolocate);

				return;
			}

			// Try to render the google maps
			EasySocial.require()
			.script($.getScript('https://maps.google.com/maps/api/js?sensor=true&key=' + ck.settings('googleapi')))
			.done(function() {
				navigator.geolocation.getCurrentPosition(self.geolocate);
			});
		}
	},

	"{removeLocation} click": function(element, event) {
		self.clearLocation();
	},

	"{self} onRenderMessage": function(element, event, conversationId, userId) {
		opts.conversationId = conversationId;
		opts.recipientId = userId;

		// hide any error message if any
		self.warningWrapper().hide();
		self.reset();
	},

	"{textarea} keypress": function(element, event) {
		var keycode = (event.keyCode ? event.keyCode : event.which);
		var message = element.val();

		// Capture enter key
		if ((keycode == 13 && !event.shiftKey && ck.settings('submit') == 1) || (keycode == 10 && !event.shiftKey && event.ctrlKey && ck.settings('submit') == 1)) {
			event.preventDefault();
			event.stopPropagation();

			self.send(message);

			// Form needs to be reset after sending the message or it will mess up attachments and location.
			self.reset();

			return false;
		}
	},

	"{textarea} focus": function() {

		self.composer().addClass('is-focus');
		self.toggleTextareFocus(true);
		parent.scrollToBottom();
	},

	"{contents} click": function(element, event) {
		self.composer().removeClass('is-focus');
	},

	"{textarea} blur": function(element, event) {
		self.toggleTextareFocus(false);
	},

	toggleTextareFocus: function(toggle) {
		if (ck.platform.isIOS()) {
			version = ck.platform.getIOSVersion(true);
			if (version == 11 && ck.platform.isSafari()) {
				if (toggle) {
					ck.window().addClass('is-ios-textarea-focus');
				} else {
					ck.window().removeClass('is-ios-textarea-focus');
				}

			}
		}
	},

	"{submit} click": function(element, event) {
		event.preventDefault();

		var message = self.textarea().val();

		self.send(message);
	}
}});

module.resolve();

});
﻿EasySocial.module('ck/sound', function($) {

var module = this;

/**
 * Ion.Sound
 * version 3.0.7 Build 89
 * © Denis Ineshin, 2016
 *
 * Project page:    http://ionden.com/a/plugins/ion.sound/en.html
 * GitHub page:     https://github.com/IonDen/ion.sound
 *
 * Released under MIT licence:
 * http://ionden.com/a/plugins/licence-en.html
 */

;(function (window, navigator, $, undefined) {
	"use strict";

	window.ion = window.ion || {};

	if (ion.sound) {
		return;
	}

	var warn = function (text) {
		if (!text) text = "undefined";

		if (window.console) {
			if (console.warn && typeof console.warn === "function") {
				console.warn(text);
			} else if (console.log && typeof console.log === "function") {
				console.log(text);
			}

			var d = $ && $("#debug");
			if (d && d.length) {
				var a = d.html();
				d.html(a + text + '<br/>');
			}
		}
	};

	var extend = function (parent, child) {
		var prop;
		child = child || {};

		for (prop in parent) {
			if (parent.hasOwnProperty(prop)) {
				child[prop] = parent[prop];
			}
		}

		return child;
	};



	/**
	 * DISABLE for unsupported browsers
	 */

	if (typeof Audio !== "function" && typeof Audio !== "object") {
		var func = function () {
			warn("HTML5 Audio is not supported in this browser");
		};
		ion.sound = func;
		ion.sound.play = func;
		ion.sound.stop = func;
		ion.sound.pause = func;
		ion.sound.preload = func;
		ion.sound.destroy = func;
		func();
		return;
	}



	/**
	 * CORE
	 * - creating sounds collection
	 * - public methods
	 */

	var is_iOS = /iPad|iPhone|iPod/.test(navigator.appVersion),
		sounds_num = 0,
		settings = {},
		sounds = {},
		i;



	if (!settings.supported && is_iOS) {
		settings.supported = ["mp3", "mp4", "aac"];
	} else if (!settings.supported) {
		settings.supported = ["mp3", "ogg", "mp4", "aac", "wav"];
	}

	var createSound = function (obj) {
		var name = obj.alias || obj.name;

		if (!sounds[name]) {
			sounds[name] = new Sound(obj);
			sounds[name].init();
		}
	};

	ion.sound = function (options) {
		extend(options, settings);

		settings.path = settings.path || "";
		settings.volume = settings.volume || 1;
		settings.preload = settings.preload || false;
		settings.multiplay = settings.multiplay || false;
		settings.loop = settings.loop || false;
		settings.sprite = settings.sprite || null;
		settings.scope = settings.scope || null;
		settings.ready_callback = settings.ready_callback || null;
		settings.ended_callback = settings.ended_callback || null;

		sounds_num = settings.sounds.length;

		if (!sounds_num) {
			warn("No sound-files provided!");
			return;
		}

		for (i = 0; i < sounds_num; i++) {
			createSound(settings.sounds[i]);
		}
	};

	ion.sound.VERSION = "3.0.7";

	ion.sound._method = function (method, name, options) {
		if (name) {
			sounds[name] && sounds[name][method](options);
		} else {
			for (i in sounds) {
				if (!sounds.hasOwnProperty(i) || !sounds[i]) {
					continue;
				}

				sounds[i][method](options);
			}
		}
	};

	ion.sound.preload = function (name, options) {
		options = options || {};
		extend({preload: true}, options);

		ion.sound._method("init", name, options);
	};

	ion.sound.destroy = function (name) {
		ion.sound._method("destroy", name);

		if (name) {
			sounds[name] = null;
		} else {
			for (i in sounds) {
				if (!sounds.hasOwnProperty(i)) {
					continue;
				}
				if (sounds[i]) {
					sounds[i] = null;
				}
			}
		}
	};

	ion.sound.play = function (name, options) {
		ion.sound._method("play", name, options);
	};

	ion.sound.stop = function (name, options) {
		ion.sound._method("stop", name, options);
	};

	ion.sound.pause = function (name, options) {
		ion.sound._method("pause", name, options);
	};

	ion.sound.volume = function (name, options) {
		ion.sound._method("volume", name, options);
	};

	if ($) {
		$.ionSound = ion.sound;
	}



	/**
	 * Web Audio API core
	 * - for most advanced browsers
	 */

	var AudioContext = window.AudioContext || window.webkitAudioContext,
		audio;

	if (AudioContext) {
		audio = new AudioContext();
	}


	var Sound = function (options) {
		this.options = extend(settings);
		delete this.options.sounds;
		extend(options, this.options);

		this.request = null;
		this.streams = {};
		this.result = {};
		this.ext = 0;
		this.url = "";

		this.loaded = false;
		this.decoded = false;
		this.no_file = false;
		this.autoplay = false;
	};

	Sound.prototype = {
		init: function (options) {
			if (options) {
				extend(options, this.options);
			}

			if (this.options.preload) {
				this.load();
			}
		},

		destroy: function () {
			var stream;

			for (i in this.streams) {
				stream = this.streams[i];

				if (stream) {
					stream.destroy();
					stream = null;
				}
			}
			this.streams = {};
			this.result = null;
			this.options.buffer = null;
			this.options = null;

			if (this.request) {
				this.request.removeEventListener("load", this.ready.bind(this), false);
				this.request.removeEventListener("error", this.error.bind(this), false);
				this.request.abort();
				this.request = null;
			}
		},

		createUrl: function () {
			var no_cache = new Date().valueOf();
			this.url = this.options.path + encodeURIComponent(this.options.name) + "." + this.options.supported[this.ext] + "?" + no_cache;
		},

		load: function () {
			if (this.no_file) {
				warn("No sources for \"" + this.options.name + "\" sound :(");
				return;
			}

			if (this.request) {
				return;
			}

			this.createUrl();

			this.request = new XMLHttpRequest();
			this.request.open("GET", this.url, true);
			this.request.responseType = "arraybuffer";
			this.request.addEventListener("load", this.ready.bind(this), false);
			this.request.addEventListener("error", this.error.bind(this), false);

			this.request.send();
		},

		reload: function () {
			this.ext++;

			if (this.options.supported[this.ext]) {
				this.load();
			} else {
				this.no_file = true;
				warn("No sources for \"" + this.options.name + "\" sound :(");
			}
		},

		ready: function (data) {
			this.result = data.target;

			if (this.result.readyState !== 4) {
				this.reload();
				return;
			}

			if (this.result.status !== 200 && this.result.status !== 0) {
				warn(this.url + " was not found on server!");
				this.reload();
				return;
			}

			this.request.removeEventListener("load", this.ready.bind(this), false);
			this.request.removeEventListener("error", this.error.bind(this), false);
			this.request = null;
			this.loaded = true;
			//warn("Loaded: " + this.options.name + "." + settings.supported[this.ext]);

			this.decode();
		},

		decode: function () {
			if (!audio) {
				return;
			}

			audio.decodeAudioData(this.result.response, this.setBuffer.bind(this), this.error.bind(this));
		},

		setBuffer: function (buffer) {
			this.options.buffer = buffer;
			this.decoded = true;
			//warn("Decoded: " + this.options.name + "." + settings.supported[this.ext]);

			var config = {
				name: this.options.name,
				alias: this.options.alias,
				ext: this.options.supported[this.ext],
				duration: this.options.buffer.duration
			};

			if (this.options.ready_callback && typeof this.options.ready_callback === "function") {
				this.options.ready_callback.call(this.options.scope, config);
			}

			if (this.options.sprite) {

				for (i in this.options.sprite) {
					this.options.start = this.options.sprite[i][0];
					this.options.end = this.options.sprite[i][1];
					this.streams[i] = new Stream(this.options, i);
				}

			} else {

				this.streams[0] = new Stream(this.options);

			}

			if (this.autoplay) {
				this.autoplay = false;
				this.play();
			}
		},

		error: function () {
			this.reload();
		},

		play: function (options) {
			delete this.options.part;

			if (options) {
				extend(options, this.options);
			}

			if (!this.loaded) {
				this.autoplay = true;
				this.load();

				return;
			}

			if (this.no_file || !this.decoded) {
				return;
			}

			if (this.options.sprite) {
				if (this.options.part) {
					this.streams[this.options.part].play(this.options);
				} else {
					for (i in this.options.sprite) {
						this.streams[i].play(this.options);
					}
				}
			} else {
				this.streams[0].play(this.options);
			}
		},

		stop: function (options) {
			if (this.options.sprite) {

				if (options) {
					this.streams[options.part].stop();
				} else {
					for (i in this.options.sprite) {
						this.streams[i].stop();
					}
				}

			} else {
				this.streams[0].stop();
			}
		},

		pause: function (options) {
			if (this.options.sprite) {

				if (options) {
					this.streams[options.part].pause();
				} else {
					for (i in this.options.sprite) {
						this.streams[i].pause();
					}
				}

			} else {
				this.streams[0].pause();
			}
		},

		volume: function (options) {
			var stream;

			if (options) {
				extend(options, this.options);
			} else {
				return;
			}

			if (this.options.sprite) {
				if (this.options.part) {
					stream = this.streams[this.options.part];
					stream && stream.setVolume(this.options);
				} else {
					for (i in this.options.sprite) {
						stream = this.streams[i];
						stream && stream.setVolume(this.options);
					}
				}
			} else {
				stream = this.streams[0];
				stream && stream.setVolume(this.options);
			}
		}
	};



	var Stream = function (options, sprite_part) {
		this.alias = options.alias;
		this.name = options.name;
		this.sprite_part = sprite_part;

		this.buffer = options.buffer;
		this.start = options.start || 0;
		this.end = options.end || this.buffer.duration;
		this.multiplay = options.multiplay || false;
		this.volume = options.volume || 1;
		this.scope = options.scope;
		this.ended_callback = options.ended_callback;

		this.setLoop(options);

		this.source = null;
		this.gain = null;
		this.playing = false;
		this.paused = false;

		this.time_started = 0;
		this.time_ended = 0;
		this.time_played = 0;
		this.time_offset = 0;
	};

	Stream.prototype = {
		destroy: function () {
			this.stop();

			this.buffer = null;
			this.source = null;

			this.gain && this.gain.disconnect();
			this.source && this.source.disconnect();
			this.gain = null;
			this.source = null;
		},

		setLoop: function (options) {
			if (options.loop === true) {
				this.loop = 9999999;
			} else if (typeof options.loop === "number") {
				this.loop = +options.loop - 1;
			} else {
				this.loop = false;
			}
		},

		update: function (options) {
			this.setLoop(options);
			if ("volume" in options) {
				this.volume = options.volume;
			}
		},

		play: function (options) {
			if (options) {
				this.update(options);
			}

			if (!this.multiplay && this.playing) {
				return;
			}

			this.gain = audio.createGain();
			this.source = audio.createBufferSource();
			this.source.buffer = this.buffer;
			this.source.connect(this.gain);
			this.gain.connect(audio.destination);
			this.gain.gain.value = this.volume;

			this.source.onended = this.ended.bind(this);

			this._play();
		},

		_play: function () {
			var start,
				end;

			if (this.paused) {
				start = this.start + this.time_offset;
				end = this.end - this.time_offset;
			} else {
				start = this.start;
				end = this.end;
			}

			if (end <= 0) {
				this.clear();
				return;
			}

			if (typeof this.source.start === "function") {
				this.source.start(0, start, end);
			} else {
				this.source.noteOn(0, start, end);
			}

			this.playing = true;
			this.paused = false;
			this.time_started = new Date().valueOf();
		},

		stop: function () {
			if (this.playing && this.source) {
				if (typeof this.source.stop === "function") {
					this.source.stop(0);
				} else {
					this.source.noteOff(0);
				}
			}

			this.clear();
		},

		pause: function () {
			if (this.paused) {
				this.play();
				return;
			}

			if (!this.playing) {
				return;
			}

			this.source && this.source.stop(0);
			this.paused = true;
		},

		ended: function () {
			this.playing = false;
			this.time_ended = new Date().valueOf();
			this.time_played = (this.time_ended - this.time_started) / 1000;
			this.time_offset += this.time_played;

			if (this.time_offset >= this.end || this.end - this.time_offset < 0.015) {
				this._ended();
				this.clear();

				if (this.loop) {
					this.loop--;
					this.play();
				}
			}
		},

		_ended: function () {
			var config = {
				name: this.name,
				alias: this.alias,
				part: this.sprite_part,
				start: this.start,
				duration: this.end
			};

			if (this.ended_callback && typeof this.ended_callback === "function") {
				this.ended_callback.call(this.scope, config);
			}
		},

		clear: function () {
			this.time_played = 0;
			this.time_offset = 0;
			this.paused = false;
			this.playing = false;
		},

		setVolume: function (options) {
			this.volume = options.volume;

			if (this.gain) {
				this.gain.gain.value = this.volume;
			}
		}
	};

	if (audio) {
		return;
	}



	/**
	 * Fallback for HTML5 audio
	 * - for not so modern browsers
	 */

	var checkSupport = function () {
		var sound = new Audio(),
			can_play_mp3 = sound.canPlayType('audio/mpeg'),
			can_play_ogg = sound.canPlayType('audio/ogg'),
			can_play_aac = sound.canPlayType('audio/mp4; codecs="mp4a.40.2"'),
			item, i;

		for (i = 0; i < settings.supported.length; i++) {
			item = settings.supported[i];

			if (!can_play_mp3 && item === "mp3") {
				settings.supported.splice(i, 1);
			}

			if (!can_play_ogg && item === "ogg") {
				settings.supported.splice(i, 1);
			}

			if (!can_play_aac && item === "aac") {
				settings.supported.splice(i, 1);
			}

			if (!can_play_aac && item === "mp4") {
				settings.supported.splice(i, 1);
			}
		}

		sound = null;
	};
	checkSupport();



	Sound.prototype = {
		init: function (options) {
			if (options) {
				extend(options, this.options);
			}

			this.inited = true;

			if (this.options.preload) {
				this.load();
			}
		},

		destroy: function () {
			var stream;

			for (i in this.streams) {
				stream = this.streams[i];

				if (stream) {
					stream.destroy();
					stream = null;
				}
			}
			this.streams = {};
			this.loaded = false;
			this.inited = false;
		},

		load: function () {
			var part;

			this.options.preload = true;
			this.options._ready = this.ready;
			this.options._scope = this;

			if (this.options.sprite) {

				for (i in this.options.sprite) {
					part = this.options.sprite[i];

					this.options.start = part[0];
					this.options.end = part[1];

					this.streams[i] = new Stream(this.options, i);
				}

			} else {

				this.streams[0] = new Stream(this.options);

			}
		},

		ready: function (duration) {
			if (this.loaded) {
				return;
			}

			this.loaded = true;

			var config = {
				name: this.options.name,
				alias: this.options.alias,
				ext: this.options.supported[this.ext],
				duration: duration
			};

			if (this.options.ready_callback && typeof this.options.ready_callback === "function") {
				this.options.ready_callback.call(this.options.scope, config);
			}

			if (this.autoplay) {
				this.autoplay = false;
				this.play();
			}
		},

		play: function (options) {
			if (!this.inited) {
				return;
			}

			delete this.options.part;

			if (options) {
				extend(options, this.options);
			}

			console.log(1);
			if (!this.loaded) {
				if (!this.options.preload) {
					this.autoplay = true;
					this.load();
				} else {
					this.autoplay = true;
				}

				return;
			}

			if (this.options.sprite) {
				if (this.options.part) {
					this.streams[this.options.part].play(this.options);
				} else {
					for (i in this.options.sprite) {
						this.streams[i].play(this.options);
					}
				}
			} else {
				this.streams[0].play(this.options);
			}
		},

		stop: function (options) {
			if (!this.inited) {
				return;
			}

			if (this.options.sprite) {

				if (options) {
					this.streams[options.part].stop();
				} else {
					for (i in this.options.sprite) {
						this.streams[i].stop();
					}
				}

			} else {
				this.streams[0].stop();
			}
		},

		pause: function (options) {
			if (!this.inited) {
				return;
			}

			if (this.options.sprite) {

				if (options) {
					this.streams[options.part].pause();
				} else {
					for (i in this.options.sprite) {
						this.streams[i].pause();
					}
				}

			} else {
				this.streams[0].pause();
			}
		},

		volume: function (options) {
			var stream;

			if (options) {
				extend(options, this.options);
			} else {
				return;
			}

			if (this.options.sprite) {
				if (this.options.part) {
					stream = this.streams[this.options.part];
					stream && stream.setVolume(this.options);
				} else {
					for (i in this.options.sprite) {
						stream = this.streams[i];
						stream && stream.setVolume(this.options);
					}
				}
			} else {
				stream = this.streams[0];
				stream && stream.setVolume(this.options);
			}
		}
	};



	Stream = function (options, sprite_part) {
		this.name = options.name;
		this.alias = options.alias;
		this.sprite_part = sprite_part;

		this.multiplay = options.multiplay;
		this.volume = options.volume;
		this.preload = options.preload;
		this.path = settings.path;
		this.start = options.start || 0;
		this.end = options.end || 0;
		this.scope = options.scope;
		this.ended_callback = options.ended_callback;

		this._scope = options._scope;
		this._ready = options._ready;

		this.setLoop(options);

		this.sound = null;
		this.url = null;
		this.loaded = false;

		this.start_time = 0;
		this.paused_time = 0;
		this.played_time = 0;

		this.init();
	};

	Stream.prototype = {
		init: function () {
			this.sound = new Audio();
			this.sound.volume = this.volume;

			this.createUrl();

			this.sound.addEventListener("ended", this.ended.bind(this), false);
			this.sound.addEventListener("canplaythrough", this.can_play_through.bind(this), false);
			this.sound.addEventListener("timeupdate", this._update.bind(this), false);

			this.load();
		},

		destroy: function () {
			this.stop();

			this.sound.removeEventListener("ended", this.ended.bind(this), false);
			this.sound.removeEventListener("canplaythrough", this.can_play_through.bind(this), false);
			this.sound.removeEventListener("timeupdate", this._update.bind(this), false);

			this.sound = null;
			this.loaded = false;
		},

		createUrl: function () {
			var rand = new Date().valueOf();
			this.url = this.path + encodeURIComponent(this.name) + "." + settings.supported[0] + "?" + rand;
		},

		can_play_through: function () {
			if (this.preload) {
				this.ready();
			}
		},

		load: function () {
			this.sound.src = this.url;
			this.sound.preload = this.preload ? "auto" : "none";
			if (this.preload) {
				this.sound.load();
			}
		},

		setLoop: function (options) {
			if (options.loop === true) {
				this.loop = 9999999;
			} else if (typeof options.loop === "number") {
				this.loop = +options.loop - 1;
			} else {
				this.loop = false;
			}
		},

		update: function (options) {
			this.setLoop(options);

			if ("volume" in options) {
				this.volume = options.volume;
			}
		},

		ready: function () {
			if (this.loaded || !this.sound) {
				return;
			}

			this.loaded = true;
			this._ready.call(this._scope, this.sound.duration);

			if (!this.end) {
				this.end = this.sound.duration;
			}
		},

		play: function (options) {
			if (options) {
				this.update(options);
			}

			if (!this.multiplay && this.playing) {
				return;
			}

			this._play();
		},

		_play: function () {
			if (this.paused) {
				this.paused = false;
			} else {
				try {
					this.sound.currentTime = this.start;
				} catch (e) {}
			}

			this.playing = true;
			this.start_time = new Date().valueOf();
			this.sound.volume = this.volume;
			this.sound.play();
		},

		stop: function () {
			if (!this.playing) {
				return;
			}

			this.playing = false;
			this.paused = false;
			this.sound.pause();
			this.clear();

			try {
				this.sound.currentTime = this.start;
			} catch (e) {}
		},

		pause: function () {
			if (this.paused) {
				this._play();
			} else {
				this.playing = false;
				this.paused = true;
				this.sound.pause();
				this.paused_time = new Date().valueOf();
				this.played_time += this.paused_time - this.start_time;
			}
		},

		_update: function () {
			if (!this.start_time) {
				return;
			}

			var current_time = new Date().valueOf(),
				played_time = current_time - this.start_time,
				played = (this.played_time + played_time) / 1000;

			if (played >= this.end) {
				if (this.playing) {
					this.stop();
					this._ended();
				}
			}
		},

		ended: function () {
			if (this.playing) {
				this.stop();
				this._ended();
			}
		},

		_ended: function () {
			this.playing = false;

			var config = {
				name: this.name,
				alias: this.alias,
				part: this.sprite_part,
				start: this.start,
				duration: this.end
			};

			if (this.ended_callback && typeof this.ended_callback === "function") {
				this.ended_callback.call(this.scope, config);
			}

			if (this.loop) {
				setTimeout(this.looper.bind(this), 15);
			}
		},

		looper: function () {
			this.loop--;
			this.play();
		},

		clear: function () {
			this.start_time = 0;
			this.played_time = 0;
			this.paused_time = 0;
		},

		setVolume: function (options) {
			this.volume = options.volume;

			if (this.sound) {
				this.sound.volume = this.volume;
			}
		}
	};

} (window, navigator, window.jQuery || window.$));


module.resolve();

});