EasySocial.module('ck/monitoring', function($) {

var module = this;

EasySocial.Controller('CK.Monitoring', {
	defaultOptions: {
		interval: 1,
		wait: false
	}
}, function(self, opts, base, parent) { return {

	init: function() {
		parent = self.parent;

		// Start monitoring
		self.start();
	},

	getInterval: function() {
		var interval = parent.settings('monitorInterval') * 1000;

		// Automatically increase the interval to 5 seconds when the tab is not active
		if (document.hidden) {
			interval = 5000;
		}

		return interval;
	},

	check: function() {

		setTimeout(function() {

			if (opts.wait) {
				return;
			}

			// update the wait so that we will wait
			// for the process to complete before we fire
			// another checking.
			opts.wait = true;

			// Determines the current state of the chat
			var state = parent.isOpen() ? 'open' : 'close';

			var currentTimeStamp = null;

			if (state == 'open') {
				currentTimeStamp = parent.getCurrentTimeStamp();
			}

			$.ajax({
				"url": window.es.rootUrl + '/plugins/system/conversekit/ajax.php',
				"method": "post",
				"data": {
					"state": state,
					"userId": parent.options.userId,
					"timestamp": currentTimeStamp
				}
			}).done(function(data) {

				var timestamp = null;
				var curtimestamp = null;

				if (data.indexOf('|') > -1) {
					var tmp = data.split('|');

					data = parseInt(tmp[0]);
					timestamp = tmp[1];
					curtimestamp = tmp[2];

					parent.setCurrentTimeStamp(timestamp);
				} else {
					data = parseInt(data);
				}

				// Trigger result
				// Also, the wait state will be updated by onConverseKitUpdate
				parent.trigger('onConverseKitUpdate', [data, curtimestamp, self.getInterval()]);

				// Start the checking again
				self.start();
			});

		}, self.getInterval());
	},

	start: function() {
		opts.state = setTimeout(self.check, self.getInterval());
	},

	stop: function() {
		clearTimeout(opts.state);
	}

}});

module.resolve();

});
