<?php
/**
* @package		ConverseKit
* @copyright	Copyright (C) Stack Ideas Sdn Bhd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* ConverseKit is free software. This version may have been modified pursuant
* to the GNU General Public License, and as distributed it includes or
* is derivative of works licensed under the GNU General Public License or
* other free or open source software licenses.
* See COPYRIGHT.php for copyright notices and details.
*/
defined('_JEXEC') or die('Unauthorized Access');

jimport('joomla.filesystem.file');

// Since conversekit is dependent on EasySocial currently, we'll need the library from EasySocial
$library = JPATH_ADMINISTRATOR . '/components/com_easysocial/includes/plugins.php';

if (!JFile::exists($library)) {
	return;
}


// Include dependencies
require_once($library);
require_once(__DIR__ . '/libraries/helper.php');

class plgSystemConversekit extends EasySocialPlugins
{
	/**
	 * Load the language file on instantiation. Note this is only available in Joomla 3.1 and higher.
	 * If you want to support 3.0 series you must override the constructor
	 *
	 * @var    boolean
	 * @since  3.1
	 */
	protected $autoloadLanguage = true;

	/**
	 * Determines if ConverseKit should really be enabled for the user
	 *
	 * @since	1.0.4
	 * @access	public
	 */
	public function enabled()
	{
		// Need to ensure that conversekit is enabled
		if (!$this->config->get('conversations.enabled')) {
			return false;
		}

		// We also need to ensure that the site is not in offline mode
		if ($this->app->get('offline') && !$this->my->authorise('core.login.offline')) {
			return false;
		}

		return true;
	}

	/**
	 * Determines if we should exclude certain item from triggering this plugin.
	 *
	 * @since   1.0.0
	 * @access  public
	 */
	public function exclude()
	{
		// There are no one to chat with at the backend
		if (ES::isFromAdmin()) {
			return true;
		}

		$option = $this->input->get('option', '', 'string');
		$view = $this->input->get('view', '', 'string');

		// Only include in com_easysocial pages
		if ($option != 'com_easysocial' && $this->params->get('easysocial_only', false)) {
			return true;
		}

		// To fix drag and drop image issue in tinymce in joomla 3.5.
		if ($option == 'com_media') {
			return true;
		}

		$tmpl = $this->input->get('tmpl', false, 'default');

		if ($tmpl == 'component') {
			return true;
		}

		// Do not show conversekit on mobile devices
		if (!$this->params->get('render_mobile', true) && $this->isMobile()) {
			return true;
		}

		// Do not show conversekit if it is not enabled
		if (!$this->enabled()) {
			return true;
		}

		// Only display button when enabled
		if (!$this->params->get('display_login', true) && !$this->my->id) {
			return true;
		}

		// Determine if user really able to access the conversation
		if (!$this->my->hasCommunityAccess()) {
			return true;
		}

		if ($this->config->get('user.completeprofile.required', false) && $this->config->get('user.completeprofile.action') == 'redirect') {
			$percentage = $this->my->getProfileCompleteness();

			if ($percentage < 100) {
				return true;
			}
		}

		// Prior to ES 3.0, we no longer show ck icon in conversations page to avoid redundancy. #305
		if ($option == 'com_easysocial' && $view == 'conversations') {
			return true;
		}

		return false;
	}

	/**
	 * Triggered after the site has been rendered
	 *
	 * @since 	1.0
	 * @access 	public
	 */
	public function onAfterRender()
	{
		$doc = JFactory::getDocument();
		if ($this->exclude() || $doc->getType() != 'html') {
			return;
		}

		$model = CK::model('conversekit');
		$initialCounter = $model->getTotalUnReadConversations($this->my->id);

		$fileType = getType($this->config->get('conversations.attachments.types'));
		$file = $this->config->get('conversations.attachments.types');
		$array = array();

		//some pc will have array value for this, therefore normalise to array before pass as json_encode
		if ($fileType == 'string') {
			$array = explode(',', $file);
		}

		$fileType = json_encode($array);

		// Normalize sound & submit setting
		$settings = $model->getSettings($this->my->id, $this->params);

		$soundSettings = $settings['sound'];
		$submitSettings = $settings['submit'];
		$darkModeSettings = $settings['darkmode'];

		// Retrieve the login return url from EasySocial
		$return = CK::getLoginReturnUrl();

		// check for es mobile integration.
		$esmobileInstalled = JPluginHelper::getPlugin('system', 'easysocialmobile');
		$esmobileEnabled = JPluginHelper::isEnabled('system', 'easysocialmobile');
		$hideButton = ($esmobileInstalled && $esmobileEnabled && !CK::isDesktop()) ? true : false;

		$component = $this->input->get('option', '', 'string');

		// The button can be shown if esmobile's 'EasySocial Pages Only' enabled and CK's 'Only Render on EasySocial Pages' disabled
		if ($esmobileInstalled && $esmobileEnabled && !CK::isDesktop() && $component != 'com_easysocial' && !$this->params->get('easysocial_only', false)) {
			$params = ES::registry($esmobileInstalled->params);

			if ($params->get('easysocial_only', false)) {			
				$hideButton = false;
			}		
		}

		$conversation = ES::conversation();

		$emoji = ES::smileys()->getEmojis();
		$giphy = ES::giphy();
		$token = ES::token();
		
		$path = SOCIAL_JOOMLA_URI;

		$options = array(
						'my' => $this->my,
						'createConversation' => $conversation->canCreate(),
						'userId' => $this->my->id,
						'initialCounter' => $initialCounter,
						'soundSettings' => $soundSettings,
						'submitSettings' => $submitSettings,
						'darkModeSettings' => $darkModeSettings,
						'contactLimit' => $this->params->get('contact_limit', 20),
						'listLimit' => $this->params->get('conversation_limit', 20),
						'conversationSorting' => $this->config->get('conversations.sorting'),
						'conversationOrdering' => $this->config->get('conversations.ordering'),
						'path' => $path,
						'location' => $this->config->get('conversations.location'),
						'attachment' => $this->config->get('conversations.attachments.enabled'),
						'googleapi' => $this->config->get('location.maps.api'),
						'mapProvider' => $this->config->get('location.provider'),
						'fileType'=> $fileType,
						'fileSize'=> $this->config->get('conversations.attachments.maxsize'),
						'esToken' => $token,
						'esConfig' => $this->config,
						'monitorInterval' => $this->params->get('monitor_interval', 4),
						'remember' => $this->params->get('remember_state', false),
						'params' => $this->params,
						'showOnlineUsers' => $this->config->get('conversations.nonfriend'),
						'showOnlineFriends' => $this->config->get('friends.enabled'),
						'isIOS' => CK::isIOS(),
						'sso' => ES::sso(),
						'returnUrl' => $return,
						'hideButton' => $hideButton,
						'smileys' => $emoji,
						'giphyEnabledForConversations' => $giphy->isEnabledForConversations(),
						'component' => $component
					);

		$templateContents = CK::getTemplateContents('default', $options);

		$body = $this->app->getBody();
		$body = preg_replace('/\<(\s*?)\/(\s*?)body\>/is', $templateContents . '</body>', $body);

		$this->app->setBody($body);
	}

	/**
	 * The main entry point for conversekit to attach itself
	 *
	 * @since	1.0
	 * @access	public
	 */
	public function onAfterDispatch()
	{
		$doc = JFactory::getDocument();
		if ($this->exclude() || $doc->getType() != 'html') {
			return;
		}

		// Headers would need to be loaded after application is loaded
		CK::initialize($this->params);
	}

	/**
	 * Intercepts ajax requests for conversekit
	 *
	 * @since	1.0.0
	 * @access	public
	 */
	public function onAfterRoute()
	{
		$ajax = $this->input->get('conversekit', false, 'bool');
		$api = $this->input->get('api', false, 'bool');
		$format = $this->input->get('format', '', 'default');

		// Process rest api calls
		if ($ajax && $api) {
			$this->processRestApi();
			return;
		}

		if ($this->exclude() || $format !== 'json' || !$ajax) {
			return;
		}

		// Include the main abstract file
		require_once(__DIR__ . '/libraries/easysocial.php');

		$method = $this->input->get('namespace', null, 'default');

		$lib = new ConversekitEasySocial($this->params);

		if (!method_exists($lib, $method)) {
			die('Invalid request');
		}

		$lib->$method();
		exit;
	}

	/**
	 * Process REST API Requests
	 *
	 * @since	1.2.0
	 * @access	public
	 */
	public function processRestApi()
	{
		// Include the main abstract file
		require_once(__DIR__ . '/libraries/api.php');

		$method = $this->input->get('namespace', null, 'default');

		$lib = new ConversekitApi($this->params);

		if (!method_exists($lib, $method)) {
			die('Invalid request');
		}

		$lib->$method();
		exit;
	}

	/**
	 * Injects the necessary API key to reach the download server
	 *
	 * @since	2.0.0
	 * @access	public
	 */
	public function onInstallerBeforePackageDownload(&$url, &$headers)
	{
		$app = JFactory::getApplication();

		// Match the updater site
		$match = stristr($url, 'https://services.stackideas.com/updater/conversekit') !== false;

		// Only check against our update sites
		if (!$match) {
			return true;
		}

		// Get user's subscription key
		if (!CK_KEY) {
			$app->enqueueMessage('Your setup contains an invalid api key and ConverseKit will not be updated now. If the problem still persists, please get in touch with the support team at https://stackideas.com/forums', 'error');

			return true;
		}

		$domain = str_ireplace(array('http://', 'https://'), '', rtrim(JURI::root(), '/'));
		$localVersion = CK::getVersion();

		$uri = new JURI($url);
		$uri->setVar('from', $localVersion);
		$uri->setVar('key', CK_KEY);
		$uri->setVar('domain', $domain);
		$url = $uri->toString();

		return true;
	}
}
