<?php
/**
* @package		ConverseKit
* @copyright	Copyright (C) Stack Ideas Sdn Bhd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* ConverseKit is free software. This version may have been modified pursuant
* to the GNU General Public License, and as distributed it includes or
* is derivative of works licensed under the GNU General Public License or
* other free or open source software licenses.
* See COPYRIGHT.php for copyright notices and details.
*/
defined('_JEXEC') or die('Unauthorized Access');

class CKMinifier
{
	const URL = 'http://deployer.stackideas.com:1280';

	/**
	 * Minifies an uncompressed javascript source
	 *
	 * @since	1.1.0
	 * @access	public
	 */
	public function minify($contents, array $options = array())
	{
		$post = $this->build($contents);

		$bytes = strlen($post);

		if (function_exists('mb_strlen') && ((int)ini_get('mbstring.func_overload') & 2)) {
			$bytes = mb_strlen($post, '8bit');
		}

		$response = $this->getResponse($post);

		if (preg_match('/^Error\(\d\d?\):/', $response)) {
			throw new Exception($response, 400);
		}


		if ($response === '') {
			$errors = $this->getResponse($this->build($contents, true));

			throw new Exception($errors, 400);
		}

		return $response;
	}

	/**
	 * Sends the data to the compiler
	 *
	 * @since	1.1.0
	 * @access	public
	 */
	protected function getResponse($post)
	{
		$ch = curl_init(self::URL);
		curl_setopt($ch, CURLOPT_POST, true);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($ch, CURLOPT_HTTPHEADER, array('Content-type: application/x-www-form-urlencoded'));
		curl_setopt($ch, CURLOPT_POSTFIELDS, $post);
		curl_setopt($ch, CURLOPT_FOLLOWLOCATION, false);
		curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 30);

		$contents = curl_exec($ch);
		curl_close($ch);

		if (false === $contents) {
			throw new Exception('No HTTP response from compiler server', 500);
		}

		return trim($contents);
	}

	/**
	 * Generates the post body that needs to be sent to the server
	 *
	 * @since	1.1.0
	 * @access	public
	 */
	protected function build($js, $returnErrors = false)
	{
		return http_build_query(array(
			'js_code' => $js,
			'output_info' => ($returnErrors ? 'errors' : 'compiled_code'),
			'output_format' => 'text',
			'compilation_level' => 'SIMPLE_OPTIMIZATIONS',
			'language'   => 'ECMASCRIPT5'
		), null, '&');
	}
}
