<?php
/**
* @package		ConverseKit
* @copyright	Copyright (C) Stack Ideas Sdn Bhd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* ConverseKit is free software. This version may have been modified pursuant
* to the GNU General Public License, and as distributed it includes or
* is derivative of works licensed under the GNU General Public License or
* other free or open source software licenses.
* See COPYRIGHT.php for copyright notices and details.
*/
defined('_JEXEC') or die('Unauthorized Access');

class ConversekitModelConverseKit
{
	// EasySocial conversation library
	private $lib = null;

	private $total = 0;

	public function __construct()
	{
		$this->lib = ES::conversations();
	}

	/**
	 * Try to retrieve the conversation id (if there are any conversations), given two different user ids.
	 *
	 * @since   2.0.0
	 * @access  public
	 */
	public function getConversationId($current, $target)
	{
		$conversation = ES::table('Conversation');
		$exists = $conversation->loadByRelation($current, $target, SOCIAL_CONVERSATION_SINGLE);

		if ($conversation->id) {
			return $conversation->id;
		}

		return false;
	}

	/**
	 * Retrieves new notification data
	 *
	 * @since	1.2.0
	 * @access	public
	 */
	public function getNewMessages($conversationId, $userId, $timestamp, $exclusion = array(), $options = array())
	{
		$db = ES::db();
		$isSSE = isset($options['single_polling']) ? $options['single_polling'] : false;
		$interval = isset($options['monitor_interval']) ? $options['monitor_interval'] : 5;

		$query = [
			'SELECT a.*, a.`created` AS `lastmodif`, b.`user_id`, b.`message_id`',
			'FROM `#__social_conversations_message` AS a',
			'INNER JOIN `#__social_conversations_message_maps` AS b',
			'ON a.`id` = b.`message_id`',
			'WHERE a.`conversation_id`=' . $db->Quote($conversationId),
			'AND b.`user_id`=' . $db->Quote($userId),
			'AND b.`state`= 1',
			'AND b.`isread`= 0',
			'AND a.`created_by` !=' . $db->Quote($userId),
		];

		$created = 'AND a.`created` >= ' . $db->Quote($timestamp);

		if (!$isSSE) {
			$created = 'AND a.`created` >= ' . 'DATE_SUB(' . $db->Quote($timestamp) . ', INTERVAL ' . $interval . ' SECOND)';
		}

		$query[] = $created;

		// Exclusion list
		if ($exclusion) {
			$query[] = 'AND a.`id` NOT IN(' . implode(',', $exclusion) . ')';
		}

		// debug
		// var_dump(implode(' ', $query));exit;

		$db->setQuery($query);

		$data = $db->loadObjectList();

		return $data;
	}

	/**
	 * Determines if a conversation exists between 2 users
	 *
	 * @since	1.1.0
	 * @access	public
	 */
	public function getSingleConversation($sourceId, $targetId)
	{
		$table = ES::table('Conversation');
		$table->loadByRelation($sourceId, $targetId, SOCIAL_CONVERSATION_SINGLE);

		return $table;
	}

	/**
	 * Generates total number of unread messages by the user
	 *
	 * @since	1.1.0
	 * @access	public
	 */
	public function getTotalUnReadConversations($userId)
	{
		$db = JFactory::getDBO();
		$userId = (int) $userId;

		$query = array();

		$query = "select count(a.`conversation_id`) as `total`";
		$query .= " from `#__social_conversations_message` as a";
		$query .= "		inner join `#__social_conversations_message_maps` as b on a.`id` = b.`message_id`";
		$query .= " 	inner join `#__users` as c on a.`created_by` = c.`id`";
		$query .= " where b.`user_id` = " . $db->Quote($userId);
		$query .= " and b.`state` = 1";
		$query .= " and b.`isread` = 0";
		$query .= " and a.`created_by` != " . $db->Quote($userId);

		$db->setQuery($query);
		$total = (int) $db->loadResult();

		return $total;
	}

	/**
	 * Retrieves the settings for a given user
	 *
	 * @since	1.1.0
	 * @access	public
	 */
	public function getSettings($userId, JRegistry $params)
	{
		static $cache = array();

		if (!isset($cache[$userId])) {
			$db = JFactory::getDbo();
			$query = $db->getQuery(true);

			$query->select($db->quoteName('setting'))
				->from($db->quoteName('#__conversekit_setting'))
				->where($db->quoteName('user_id').'=' . $db->quote($userId));

			$db->setQuery($query);
			$result = $db->loadResult();

			$settings = @unserialize($result);

			if ($settings === false) {
				$settings = array(
									'sound' => $params->get('sound_setting', true),
									'submit' => $params->get('submit_setting', true),
									'darkmode' => $params->get('darkmode_setting', false),
									'app' => false
								);
			}

			if (!isset($settings['app'])) {
				$settings['app'] = false;
			}
			
			if (!isset($settings['darkmode'])) {
				$settings['darkmode'] = $params->get('darkmode_setting', false);
			}

			$cache[$userId] = $settings;
		}

		return $cache[$userId];
	}

	/**
	 * Stores settings for a given user
	 *
	 * @since	1.1.0
	 * @access	public
	 */
	public function setSettings($userId, $setting)
	{
		$db = JFactory::getDbo();

		$query = "insert into `#__conversekit_setting` (" . $db->qn('user_id') . ", " .  $db->qn('setting') . ") values (" . $db->Quote($userId) . ", " . $db->Quote($setting) . ")";
		$query .= " ON DUPLICATE KEY UPDATE `setting` = " . $db->Quote($setting);

		$db->setQuery($query);
		$state = $db->execute();

		return $state;
	}

	/**
	 * Retrieves new notification data
	 *
	 * @since	1.1.0
	 * @access	public
	 */
	public function getNotification($userId, $timestamp = null, $conversationId = null)
	{
		$db = JFactory::getDbo();
		$data = array();

		$query = "select a.*, a.`created` AS `lastmodif`, b.`user_id`, b.`message_id`";
		$query .= " from `#__social_conversations_message` as a";
		$query .= "		inner join `#__social_conversations_message_maps` as b on a.`id` = b.`message_id`";
		$query .= " 	inner join `#__users` as c on a.`created_by` = c.`id`";
		$query .= " where b.`user_id` = " . $db->Quote($userId);
		if ($conversationId) {
			$query .= " and a.`conversation_id` = " . $db->Quote($conversationId);
		}
		$query .= " and b.`state` = 1";
		$query .= " and b.`isread` = 0";
		$query .= " and a.`created_by` != " . $db->Quote($userId);

		if ($timestamp) {
			$query .= " and a.`created` >= ". $db->Quote($timestamp);
		}

		$db->setQuery($query);


		$results = $db->loadObjectList();

		if ($results) {

			// manually group the new message count for each conversation
			$newMsgCount = array();

			foreach ($results as $result) {
				$newMsgCount[$result->conversation_id][] = 1;
			}

			// now let 'attach' the new message count into th resultset
			for ($i = 0; $i < count($results); $i++) {
				$row = $results[$i];
				$row->newMsg = count($newMsgCount[$result->conversation_id]);

				$data[] = $row;
			}

		}

		return $data;
	}

	/**
	 * Get messages for a particular conversation
	 *
	 * @since   1.1.0
	 * @access  public
	 */
	public function getMessages($userId, $conversationId, $options = array())
	{
		$config = ES::config();
		$db = ES::db();
		$sql = $db->sql();
		$query = array();

		$query[] = 'SELECT a.*,c.latitude,c.longitude,c.id AS location_id ,c.address,';
		$query[] = ' FLOOR((UNIX_TIMESTAMP(now()) - UNIX_TIMESTAMP(a.' . $db->nameQuote('created') . ')) / 60 / 60 / 24) AS ' . $db->nameQuote('day');
		$query[] = 'FROM ' . $db->nameQuote('#__social_conversations_message') . ' AS a';
		$query[] = 'INNER JOIN ' . $db->nameQuote('#__social_conversations_message_maps') . ' AS b';
		$query[] = 'ON b.' . $db->nameQuote('message_id') . ' = a.' . $db->nameQuote('id');
		$query[] = ' LEFT JOIN ' . $db->nameQuote('#__social_locations') . ' as c';
		$query[] = ' ON a.' . $db->nameQuote('id') . ' = c.' . $db->nameQuote('uid') ;
		$query[] = ' 	and  c.' . $db->nameQuote('type') . ' = ' . $db->Quote('conversations') ;

		if ($config->get('users.blocking.enabled') && !JFactory::getUser()->guest) {
			// user block
			$query[] = ' LEFT JOIN ' . $db->nameQuote('#__social_block_users') . ' as bus';
			$query[] = ' ON a.' . $db->nameQuote('created_by') . ' = bus.' . $db->nameQuote('user_id') ;
			$query[] = ' AND bus.' . $db->nameQuote('target_id') . ' = ' . $db->Quote(JFactory::getUser()->id) ;
		}

		$query[] = 'WHERE a.' . $db->nameQuote('conversation_id') . '=' . $db->Quote($conversationId);
		$query[] = 'AND b.' . $db->nameQuote('user_id') . '=' . $db->Quote($userId);

		if ($config->get('users.blocking.enabled') && !JFactory::getUser()->guest) {
			// user block continue here
			$query[] = ' AND bus.' . $db->nameQuote('id') . ' IS NULL';
		}

		// Glue back query.
		$query = implode(' ' , $query);


		// get total records
		$queryCount = str_ireplace('SELECT a.*', 'SELECT COUNT(1)', $query);
		$db->setQuery($queryCount);
		$total = $db->loadResult();


		// now we add the order by here.
		$query .= ' ORDER BY a.' . $db->nameQuote('id') . ' DESC';


		// check if we need to override the limit / limitstart or not
		$limit = isset($options['limit']) ? $options['limit'] : 0;
		if ($limit) {
			$limitstart = isset($options['start']) ? $options['start'] : 0;
			$query .= " LIMIT $limitstart, $limit";
		}

		$db->setQuery($query);
		$rows = $db->loadObjectList();

		if (!$rows) {
			return $rows;
		}

		// Reverse the result.
		$rows = array_reverse($rows);

		$messages= array();
		foreach ($rows as $row) {
			$message = ES::table('ConversationMessage');
			$message->bind($row);

			// we need this day value to 'group' by date.
			$message->day = $row->day;
			$messages[] = $message;
		}

		$restApiKey = isset($options['restApi']) ? $options['restApi'] : false;
		$counter = count($messages);

		$data = $this->buildMessages($conversationId, $total, $messages, $messages[$counter-1]->created, $restApiKey);

		return $data;
	}

	/**
	 * Get messages for a particular conversation
	 *
	 * @since   2.0.0
	 * @access  public
	 */
	public function loadMore($userId, $conversationId, $options = array())
	{
		$config = ES::config();
		$db = ES::db();
		$sql = $db->sql();
		$query = array();

		$limit = $options['limit'];
		$timestamp = $options['timestamp'];

		$query[] = 'SELECT a.*,c.latitude,c.longitude,c.id AS location_id ,c.address,';
		$query[] = ' FLOOR((UNIX_TIMESTAMP(now()) - UNIX_TIMESTAMP(a.' . $db->nameQuote('created') . ')) / 60 / 60 / 24) AS ' . $db->nameQuote('day');
		$query[] = 'FROM ' . $db->nameQuote('#__social_conversations_message') . ' AS a';
		$query[] = 'INNER JOIN ' . $db->nameQuote('#__social_conversations_message_maps') . ' AS b';
		$query[] = 'ON b.' . $db->nameQuote('message_id') . ' = a.' . $db->nameQuote('id');
		$query[] = ' LEFT JOIN ' . $db->nameQuote('#__social_locations') . ' as c';
		$query[] = ' ON a.' . $db->nameQuote('id') . ' = c.' . $db->nameQuote('uid') ;
		$query[] = ' 	and  c.' . $db->nameQuote('type') . ' = ' . $db->Quote('conversations') ;
		$query[] = 'WHERE a.' . $db->nameQuote('conversation_id') . '=' . $db->Quote($conversationId);
		$query[] = 'AND b.' . $db->nameQuote('user_id') . '=' . $db->Quote($userId);
		$query[] = 'AND a.`created` < ' . $db->Quote($options['timestamp']);
		$query[] = 'ORDER BY a.`created` DESC';
		$query[] = 'LIMIT ' . $limit;

		$db->setQuery($query);
		$rows = $db->loadObjectList();

		if (!$rows) {
			return $rows;
		}

		// Reverse the result.
		$rows = array_reverse($rows);

		$messages = array();

		foreach ($rows as $row) {
			$message = ES::table('ConversationMessage');
			$message->bind($row);

			$message->day = $row->day;
			$messages[] = $message;
		}

		$counter = count($messages);
		$restApiKey = isset($options['restApiKey']) ? $options['restApiKey'] : false;

		$data = $this->formatMessages($messages, $restApiKey);

		return $data;
	}

	/**
	 * Standardize a list of messages
	 *
	 * @since	2.0.0
	 * @access	public
	 */
	public function formatMessages($items, $restApiKey = false)
	{
		$messages = array();

		if ($items) {
			foreach ($items as $item) {
				$messages[] = $this->formatMessage($item, $restApiKey);
			}
		}

		return $messages;
	}

	/**
	 * Standardize the formatting of a message that is used in the API
	 *
	 * @since	2.0.1
	 * @access	public
	 */
	public function formatMessage($item, $restApiKey = false)
	{
		$obj = $this->lib->genMessage($item);
		$obj->raw = $item->message;

		if ($obj->type == 'giphy') {
			$args = array('giphy' => $obj->raw);
			$obj->text = CK::getTemplateContents('giphy.preview', $args);
		}

		// 2.0.0 - For REST API, we need to use the absolute url.
		if ($restApiKey) {
			$obj->text = ESJString::str_ireplace('src="/', 'src="' . JURI::root(), $obj->text);
		}

		// 2.0.0 - Use displayTime property for displaying nice time
		$date = $item->getRepliedDate(false);
		$obj->displayTime = $date->format('Y-m-d H:i:s', false);

		$obj->attachments = CK::formatAttachments($obj->attachments, $restApiKey);
		
		return $obj;
	}

	/**
	 * helper method to construct json data for messages.
	 *
	 * @since   1.1.0
	 * @access  public
	 */
	private function buildMessages($conversationId, $total = 0, $messages = array(), $created = false, $restApiKey = false)
	{
		// heading object
		$data = new stdClass();
		$data->conversationId = $conversationId;
		$data->totalmessages = $total;
		$data->lastupdate = ES::date()->toSql();
		$data->timestamp = $created;
		$data->messages = $this->formatMessages($messages, $restApiKey);

		return $data;
	}

	/**
	 * Returns the total number of friend a user has.
	 *
	 * @since	1.0
	 * @access	public
	 */
	public function getTotalFriends()
	{
		// this value should be set from getFriends method.
		return $this->total;
	}

	/**
	 * Retrieve user block query for get friend query
	 *
	 * @since	2.0.3
	 * @access	public
	 */
	public function getUserBlockQuery()
	{
		$db = ES::db();
		$query = array();

		$query[] = ' LEFT JOIN ' . $db->nameQuote('#__social_block_users') . ' as bus';

		$query[] = ' ON (';
		$query[] = ' uu.' . $db->nameQuote('id') . ' = bus.' . $db->nameQuote('user_id');
		$query[] = ' AND bus.' . $db->nameQuote('target_id') . ' = ' . $db->Quote(JFactory::getUser()->id);
		$query[] = ') OR (';
		$query[] = ' uu.' . $db->nameQuote('id') . ' = bus.' . $db->nameQuote( 'target_id' ) ;
		$query[] = ' AND bus.' . $db->nameQuote('user_id') . ' = ' . $db->Quote(JFactory::getUser()->id) ;
		$query[] = ')';

		$query = implode(' ', $query);

		return $query;
	}

	/**
	 * Retrieves the query of getting friends
	 *
	 * @since	2.0.3
	 * @access	public
	 */
	public function getFriendsQuery($id, $options = array())
	{
		$config = ES::config();
		$everyone = isset($options['everyone']) ? $options['everyone'] : false;

		if (!$config->get('friends.enabled')) {
			$everyone = true;
		}

		$options['everyone'] = $everyone;

		$viewer = $id;

		$db = ES::db();
		$sql = $db->sql();

		$privacy = true;
		if (!$config->get('privacy.enabled')) {
			$privacy = false;
		}

		$query = array();

		$accessColumn = $this->getAccessColumn('access', 'uu');
		$accessCustomColumn = $this->getAccessColumn('customaccess', 'uu');

		if ($privacy) {
			$query[] = 'select * from (';
		}

		if ($everyone) {
			$query[] = 'SELECT ' . $db->Quote($id) . ' AS `actor_id`, uu.`id` AS `target_id`, uu.`id` AS `friendid`,  1 AS `state`, uu.`lastvisitDate`';
			if ($privacy) {
				$query[] = ', ' . $accessColumn . ', ' . $accessCustomColumn ;
			}

			$query[] = ' FROM ' . $db->nameQuote('#__users') . ' AS uu';

		} else {

			$query[] = 'SELECT a.*, if(a.' . $db->nameQuote('target_id') . ' = ' . $db->Quote($id) . ', a.' . $db->nameQuote('actor_id') . ', a.' . $db->nameQuote('target_id') . ') AS friendid, uu.`lastvisitDate`';
			if ($privacy) {
				$query[] = ', ' . $accessColumn . ', ' . $accessCustomColumn;
			}
			$query[] = 'FROM ' . $db->nameQuote('#__social_friends') . ' AS a';

			$query[] = 'INNER JOIN ' . $db->nameQuote('#__users') . ' AS uu';
			$query[] = 'ON uu.' . $db->nameQuote('id') . ' = if(a.' . $db->nameQuote('target_id') . ' = ' . $db->Quote($id) . ', a.' . $db->nameQuote('actor_id') . ', a.' . $db->nameQuote('target_id') . ')';

		}

		// exclude esad users
		$query[] = 'INNER JOIN `#__social_profiles_maps` as upm on uu.`id` = upm.`user_id`';
		$query[] = 'INNER JOIN `#__social_profiles` as up on upm.`profile_id` = up.`id` and up.`community_access` = 1';

		if ($config->get('users.blocking.enabled')) {
			// user block
			$query[] = $this->getUserBlockQuery();
		}

		$options['privacy'] = $privacy;
		$options['friends'] = $config->get('friends.enabled');
		$options['block'] = $config->get('users.blocking.enabled');

		// Where clause
		$query[] = $this->getCriteria($id, $options);

		$query = implode(' ', $query);

		return $query;
	}

	/**
	 * Searches for a user's friend.
	 *
	 * @since	2.0.4
	 * @access	public
	 */
	public function searchFriends($id, $name, $type, $options)
	{
		$model = ES::model('Friends');
		$friends = $model->search($id, $name, $type, $options);

		if (!$friends) {
			return false;
		}

		return $friends;
	}

	/**
	 * Retrieves the criteria for the query of getting friends
	 *
	 * @since	2.0.3
	 * @access	public
	 */
	public function getCriteria($id, $options = array())
	{
		$db = ES::db();
		$viewer = $id;

		$everyone = isset($options['everyone']) ? $options['everyone'] : false;
		$sort = isset($options['sort']) && $options['sort'] ? $options['sort'] : '';
		$limit = isset($options['limit']) ? $options['limit'] : 0;

		$query[] = 'WHERE uu.' . $db->nameQuote('block') . ' = ' . $db->Quote('0');

		$privacy = isset($options['privacy']) ? $options['privacy'] : false;
		$userBlock = isset($options['block']) ? $options['block'] : false;
		$friends = isset($options['friends']) ? $options['friends'] : false;

		// user block continue here
		if ($userBlock && !JFactory::getUser()->guest) {
			$query[] = 'AND bus.' . $db->nameQuote('id') . ' IS NULL';
		}

		if ($everyone) {
			$query[] = 'AND uu.' . $db->nameQuote('id') . ' != ' . $db->Quote($id);

		} else {

			// Check if state is passed in.
			$state = isset($options[ 'state' ]) ? $options[ 'state' ] : SOCIAL_FRIENDS_STATE_FRIENDS;
			$isRequest = isset($options[ 'isRequest' ]) ? $options[ 'isRequest' ] : false;
			$limit = isset($options[ 'limit' ]) ? $options[ 'limit' ] : false;


			$query[] = 'AND';
			$query[] = '(';
			$query[] = '(a.' . $db->nameQuote('target_id') . '=' . $db->Quote($id);
			$query[] = 'AND a.' . $db->nameQuote('state') . '=' . $db->Quote(SOCIAL_FRIENDS_STATE_FRIENDS) . ')';
			$query[] = 'OR';
			$query[] = '(a.' . $db->nameQuote('actor_id') . '=' . $db->Quote($id);
			$query[] = 'AND a.' . $db->nameQuote('state') . '=' . $db->Quote(SOCIAL_FRIENDS_STATE_FRIENDS) . ')';
			$query[] = ')';
		}

		if (isset($options['online']) && $options['online']) {

			$query[] = 'AND EXISTS (SELECT ' . $db->nameQuote('userid') . ' FROM ' . $db->nameQuote('#__session') . ' AS s WHERE s.' . $db->nameQuote('userid') . '= uu.' . $db->nameQuote('id') . ')';
		}

		// used in conversations library to retrieve users who not yet a participants in a onversation.
		$excludeIds = '';
		if (isset($options['exclude']) && $options['exclude']) {

			$excludeIds = $options['exclude'];

			if (!is_array($excludeIds)) {
				$excludeIds = explode(',', $excludeIds);
			}

			$query[] = 'AND uu.' . $db->nameQuote('id') . ' NOT IN (' . implode(',', $excludeIds) . ')';
		}


		$orderbyPrefix = 'uu';
		if ($privacy) {

			$orderbyPrefix = 'xx';

			$query[] = ') as xx';


			// privacy here.
			$query[] = " WHERE (";

			//public
			$query[] = "(xx.`access` = " . $db->Quote( SOCIAL_PRIVACY_PUBLIC ) . ") OR";

			//member
			$query[] = "( (xx.`access` = " . $db->Quote(SOCIAL_PRIVACY_MEMBER) . ") AND (" . $viewer . " > 0 ) ) OR ";

			// if allow to msg non-friends, for below privacy access, we fall back to logged user.
			if ($everyone) {

				// fall back to member
				$query[] = "( (xx.`access` IN ( " . $db->Quote(SOCIAL_PRIVACY_FRIEND) . "," . $db->Quote(SOCIAL_PRIVACY_ONLY_ME) . "," . $db->Quote(SOCIAL_PRIVACY_CUSTOM) . ")) AND (" . $viewer . " > 0 ) ) ";
			}

			if (!$everyone) {

				if ($friends) {
					//friends
					$query[] = "( (xx.`access` = " . $db->Quote(SOCIAL_PRIVACY_FRIEND) . ") AND ( (" . $this->generateIsFriendSQL( 'xx.`friendid`', $viewer ) . ") > 0 ) ) OR ";
				} else {
					// fall back to member
					$query[] = "( (xx.`access` = " . $db->Quote(SOCIAL_PRIVACY_FRIEND) . ") AND (" . $viewer . " > 0 ) ) OR ";
				}

				//only me
				$query[] = "( (xx.`access` = " . $db->Quote(SOCIAL_PRIVACY_ONLY_ME) . ") AND ( xx.`friendid` = " . $viewer . " ) ) OR ";

				// custom
				$query[] = "( (xx.`access` = " . $db->Quote(SOCIAL_PRIVACY_CUSTOM) . ") AND ( xx.`custom_access` LIKE " . $db->Quote( '%,' . $viewer . ',%' ) . "    ) )";
			}

			// end privacy
			$query[] = ")";
		}

		$query = implode(' ', $query);

		if ($sort) {
			// for now we only support lastseen sorting.
			if ($sort == 'lastseen') {
				$query .= ' order by ' . $orderbyPrefix . '.`lastvisitDate` desc';
			}
		}

		if ($limit != 0) {
			$limitstart = isset($options['start']) ? $options['start'] : 0;
			$query .= ' LIMIT ' . $limitstart . ', ' . $limit;
		}

		return $query;
	}

	/**
	 * Retrieves a list of friends
	 *
	 * @since	1.0
	 * @access	public
	 */
	public function getFriends($id, $options = array())
	{
		$db = ES::db();
		$sql = $db->sql();
		$query = $this->getFriendsQuery($id, $options);	

		$db->setQuery($query);
		$rows = $db->loadObjectList();

		if (!$rows) {
			$this->total = 0;
			return false;
		}

		$this->total = count($rows);
		
		$friends = array();

		$idONLY = isset($options['idonly']) ? true : false;

		foreach ($rows as $row) {
			if ($row->actor_id != $id) {
				$friends[] = ($idONLY) ? $row->actor_id : ES::user($row->actor_id);
			}

			if ($row->target_id != $id) {
				$friends[] = ($idONLY) ? $row->target_id : ES::user($row->target_id);
			}
		}

		return $friends;
	}

	private function generateIsFriendSQL($source, $target)
	{
		$query = "select count(1) from `#__social_friends` where ( `actor_id` = $source and `target_id` = $target) OR (`target_id` = $source and `actor_id` = $target) and `state` = 1";
		return $query;
	}


	private function getAccessColumn($type = 'access', $prefix = 'uu')
	{
		$column = '';
		if ($type == 'access') {
			$column = "(select prm.value as `access`";
			$column .= " from `#__social_privacy_map` as prm";
			$column .= "  inner join `#__social_privacy` as pp on prm.privacy_id = pp.id";
			$column .= "  left join `#__social_privacy_customize` as prc on prm.id = prc.uid and prc.utype = 'user'";
			$column .= " where prm.uid = " . $prefix . ".id and prm.utype = 'user'";
			$column .= "  and pp.type = 'profiles' and pp.rule = 'post.message'";
			$column .= " union all ";
			$column .= " select prm.value as `access`";
			$column .= " from `#__social_privacy_map` as prm";
			$column .= "  inner join `#__social_privacy` as pp on prm.privacy_id = pp.id";
			$column .= "  inner join `#__social_profiles_maps` pmp on prm.uid = pmp.profile_id";
			$column .= " where prm.utype = 'profiles' and pmp.user_id = " . $prefix . ".id";
			$column .= "  and pp.type = 'profiles' and pp.rule = 'post.message'";

			// get default privacy value on profiles.post.message
			$column .= " union all ";
			$column .= " select pp.value as `access`";
			$column .= "  from `#__social_privacy` as pp";
			$column .= "  where pp.type = 'profiles' and pp.rule = 'post.message'";

			$column .= " limit 1";
			$column .= ") as access";

		} else if ($type == 'customaccess') {

			$column = "(select concat(',', group_concat(prc.user_id SEPARATOR ','), ',') as `custom_access`";
			$column .= " from `#__social_privacy_map` as prm";
			$column .= "    inner join `#__social_privacy` as pp on prm.privacy_id = pp.id";
			$column .= "    left join `#__social_privacy_customize` as prc on prm.id = prc.uid and prc.utype = 'user'";
			$column .= " where prm.uid = " . $prefix . ".id and prm.utype = 'user'";
			$column .= "    and pp.type = 'profiles' and pp.rule = 'post.message'";
			$column .= " limit 1";
			$column .= ") as custom_access";

		}

		return $column;
	}


	/**
	 * Retrieves participants of a conversation
	 *
	 * @since	1.1.0
	 * @access	public
	 */
	public function getParticipants($userId, $conversationId, $excludeUsers = array(), $options = array())
	{
		// Ensure excluded users is an array.
		$excludeUsers = ES::makeArray($excludeUsers);

		$limit = isset($options['limit']) ? $options['limit'] : ES::themes()->getConfig()->get('friendslimit');
		$start = isset($options['start']) ? $options['start'] : 0;
		$isTotal = isset($options['totalOnly']) ? $options['totalOnly'] : false;

		$config = ES::config();
		$db = JFactory::getDbo();


		$query = "select a.`user_id`";

		if ($isTotal) {
			$query = "select count(1)";
		}

		$query .= " from `#__social_conversations_participants` as a";


		if ($config->get('users.blocking.enabled')) {
			$query .= ' LEFT JOIN `#__social_block_users` AS `bus` ON a.`user_id` = `bus`.`user_id` AND `bus`.`target_id` = ' . $db->Quote($userId);
		}

		$query .= " where a.`conversation_id` = " . $db->Quote($conversationId);
		$query .= " and a.`state` = "  . $db->Quote(SOCIAL_STATE_PUBLISHED);

		if (!empty($excludeUsers)) {
			if (is_array($excludeUsers)) {
				$query .= " and a.`user_id` NOT IN (" . implode(',', $excludeUsers) . ")";
			} else {
				$query .= " and a.`user_id` != " . $db->Quote($excludeUsers);
			}
		}

		if (!$isTotal) {
			$query .= " order by a.`id` asc";
			$query .= " LIMIT $start, $limit";
		}

		$db->setQuery($query);

		if ($isTotal) {

			$total = $db->loadResult();
			return $total;

		} else {


			// Load the data.
			$rows = $db->loadColumn();

			if (!$rows) {
				return false;
			}

			// Load the list of users.
			$users = ES::user($rows);

			return $users;

		}

	}
}
