<?php
/**
* @package		ConverseKit
* @copyright	Copyright (C) Stack Ideas Sdn Bhd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* ConverseKit is free software. This version may have been modified pursuant
* to the GNU General Public License, and as distributed it includes or
* is derivative of works licensed under the GNU General Public License or
* other free or open source software licenses.
* See COPYRIGHT.php for copyright notices and details.
*/

define('CK_BASE', dirname(dirname(dirname(dirname(__DIR__)))));

require_once(__DIR__ . '/db.php');

class ConverseKitRest
{
	protected $db = null;
	protected $debug = null;

	public function __construct()
	{
		$this->db = new Database();
		$this->debug = isset($_REQUEST['debug']) ? (bool) $_REQUEST['debug'] : false;
	}

	/**
	 * Allows user to exchange their username and password for a valid token
	 *
	 * @since	1.0.0
	 * @access	public
	 */
	protected function getUserIdFromAuthKey($key = '')
	{
		if (!$key) {
			$key = isset($_REQUEST['key']) ? $_REQUEST['key'] : '';
		}

		if (!$key) {
			return false;
		}

		static $items = array();

		if (!isset($items[$key])) {
			$items[$key] = false;

			// Check if the provided key is valid
			$query = [
				'SELECT `user_id`',
				'FROM `#__social_users` AS a',
				'WHERE a.`auth` = ' . $this->db->escape($key),
				'AND a.`state` = 1'
			];

			$result = $this->db->query($query);
			$userId = isset($result['user_id']) && $result['user_id'] ? (int) $result['user_id'] : false;

			if ($userId) {
				$items[$key] = $userId;
			}
		}

		return $items[$key];
	}

	/**
	 * Determines if the auth key is valid
	 *
	 * @since	1.0.0
	 * @access	public
	 */
	protected function isAuthKeyValid($key = '', $throwException = true)
	{
		$userId = $this->getUserIdFromAuthKey($key);

		if ($userId === false) {

			if ($throwException) {
				$this->reject('Invalid authentication key provided. Your ip will be logged');
			}

			return false;
		}

		return true;
	}

	/**
	 * Determines if this is in debug mode
	 *
	 * @since	1.0.0
	 * @access	public
	 */
	public function isDebug()
	{
		return $this->debug;
	}
}

if (!function_exists('dump')) {
	function dump()
	{
		$args = func_get_args();

		echo '<pre>';

		foreach ($args as $arg) {
			var_dump($arg);
		}
		echo '</pre>';
		exit;
	}
}

if (!function_exists('pdump')) {
	function pdump()
	{
		$args = func_get_args();

		echo '<pre>';

		foreach ($args as $arg) {
			print_r($arg);
		}
		echo '</pre>';
		exit;
	}
}