<?php
/**
* @package		ConverseKit
* @copyright	Copyright (C) Stack Ideas Sdn Bhd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* ConverseKit is free software. This version may have been modified pursuant
* to the GNU General Public License, and as distributed it includes or
* is derivative of works licensed under the GNU General Public License or
* other free or open source software licenses.
* See COPYRIGHT.php for copyright notices and details.
*/
defined('_JEXEC') or die('Unauthorized Access');

class plgSystemConversekitInstallerScript
{
	/**
	 * Triggers before the installers are copied
	 *
	 * @since	2.0.0
	 * @access	public
	 */
	public function postflight()
	{
		// Prior to ConverseKit 2.0.x, the update sites uses the title of "ConverseKit Update Site"
		// From 2.0.x onwards, we have renamed it to "ConverseKit Updater Site"
		// Hence, we need to clean up the old update sites to prevent confusions

		$this->cleanupUpdateSites();
	}

	/**
	 * Cleanup legacy update sites which are no longer in use
	 *
	 * @since	2.0.0
	 * @access	public
	 */
	public function cleanupUpdateSites()
	{
		// Delete update site with the title of "ConverseKit Update Site"
		$this->deleteUpdateSite('ConverseKit Update Site');

		// Delete update site with the title of "ConverseKit Update Site [Invalid]"
		$this->deleteUpdateSite('ConverseKit Update Site [Invalid]');
	}

	/**
	 * Workflow to delete an update site
	 *
	 * @since	2.0.0
	 * @access	public
	 */
	private function deleteUpdateSite($title)
	{
		$db = JFactory::getDBO();

		// Get the update site id first as we need to delete the relation in #__update_sites_extensions
		$query = 'SELECT `update_site_id` FROM `#__update_sites` WHERE `name`=' . $db->Quote($title);
		$db->setQuery($query);

		$id = $db->loadResult();

		if (!$id) {
			return;
		}

		// Delete update site relation from #__update_sites_extensions
		$query = 'DELETE FROM `#__update_sites_extensions` WHERE `update_site_id` = ' . $db->Quote($id);
		$db->setQuery($query);
		$db->execute();

		// Delete update site data from #__update_sites
		$query = 'DELETE FROM `#__update_sites` WHERE `update_site_id` = ' . $db->Quote($id);
		$db->setQuery($query);
		$db->execute();
	}
}
